/*
 * TITLE:		Board.c
 *
 * DESCRIPTION:	This file contains the utility functions to run various parts of the LF2407 peripherals 
 *				specific to the board.
 *
 * WRITTEN BY:	Brett Browning
 *
 * REVISION HISTORY:
 * 25 May 2001		- Created (BB)
 *
 */
/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#include "lf2407.h"
#include "board.h"
#include "adc.h"

/**************************** TYPES ********************************/


/* wathcdog check bits */
#define WD_DISABLE			(WDDIS | WDCHK)
#define WD_SET(div)			(WDCHK | ((div) & WD_PS_BITS))
#define WD_ENABLE			((~WDDIS) & WDCHK)

/* Macro to calculate the watchdog timeout period in milliseconds given divisor */
#define WDPER_CALC(y)		(((ushort) (((512L * 255L * 1000L) / CPU_FREQ))) << (y - 1))

/* motor channels */
#define MOTOR1_POS			1
#define MOTOR1_NEG			3
#define MOTOR2_POS			5
#define MOTOR2_NEG			7
#define MOTOR3_POS			9
#define MOTOR3_NEG			11


/* SCSR1 default settings 
 * - Clock at 4xFin, clock out as CPU freq, no peripherals, no osc fail reset 
 */
#define SCSR1_DEFAULT		(SCSR1_CLKO_CPU | SCSR1_PLL_4 | SCSR1_LPM0 | SCSR1_RST_OSCF)

#define MTR_DIR_CHANGE_DELAY           10

/**************************** GLOBALS *******************************/

/* io port definition */
volatile ioport ushort port0;


/**************************** CODE **********************************/


inline void fastDelay(int d);

/*
 * InitBoard -
 *
 * This function intialises the various non-specific perihperals 
 * required for the smallsize 1.00 boards. It initializes the board
 * and system registers based upon the board parameters structure.
 * wd_en, wdclk		- watch dog enable and clock setting (in ms)
 * sram_mode		- sets the mode for the internal SARAM 
 *
 * See SPRU357 appnote from www.ti.com 
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitBoard(BoardParam *bp)
{
  ushort	scsrtmp;
  ushort	div;
  ushort	per;
  ulong	tmp;
  ADCParams adcparams;
  ADCSet adcset;
  
  /* disable all interrupts */
  DI();
  
  /* should check DINR here */
  
  if (bp->wd_en) {
    
    /* calculate the watchdog divisor bits */
    for (div = 0; (WDPER_CALC(div) < bp->wdper) && (div < 7); div++)
      ;
    WD_REGS.wdcr = WD_SET(div);
    
    /* block Watchdog from ever being disabled again */
    SYS_REGS.scsr2 |= SCSR2_WDOVRIDE;
  } else {
    WD_REGS.wdcr = WD_DISABLE;
  }
  
  /* set teh system control register to default operation */	
  SYS_REGS.scsr1 = SCSR1_DEFAULT;
  
  /* set teh SARAM mode for SCSR2 - make sure we don't clear WDOVRIDE bit */
  scsrtmp = SYS_REGS.scsr2 & (~SCSR2_WDOVRIDE);
  if (bp->saram_mode & SARAM_PROG)
    scsrtmp |= SCSR2_SARAM_PROG;
  else
    scsrtmp &= ~SCSR2_SARAM_PROG;
  if (bp->saram_mode & SARAM_DATA)
    scsrtmp |= SCSR2_SARAM_DATA;
  else
    scsrtmp &= ~SCSR2_SARAM_DATA;
  SYS_REGS.scsr2 = scsrtmp;
	
  /* set the wait states */
  WSGR = WSGR_SWS_BITS(IO_WS, MEM_WS, MEM_WS) | WSGR_BUS_OFF;
  
  /* enable the LED */
  IO_REGS.pcddr &= ~(LED1_BIT | LED2_BIT | LED3_BIT);
  IO_REGS.pcddr |= LED1_DIR | LED2_DIR | LED3_DIR;
  IO_REGS.mcrb &= ~LED_MCRB;

  // push button bits
  IO_REGS.pbddr &= ~PB0_DIR;
  IO_REGS.pcddr &= ~PB1_DIR;
  IO_REGS.mcra &= ~PB_MCRA;
  IO_REGS.mcrb &= ~PB_MCRB;

  // qdec bits
  IO_REGS.pdddr |= QDEC_SEL_DIR;
  IO_REGS.pfddr |= QDEC_OE_DIR;
  QDEC_SEL_LOW();
  QDEC_OE_HI();
  IO_REGS.mcrb &= ~QDEC_MCRB;
  IO_REGS.mcrc &= ~QDEC_MCRC;

  // set the kicker bits
  IO_REGS.pfddr |= KICK_DIR | KEN_DIR;
  KICK_OFF();
  KEN_OFF();
  IO_REGS.mcrc &= ~KICK_MCRC;

  // set ball led bits
  IO_REGS.pbddr |= BALLO_DIR;
  IO_REGS.pbddr &= ~BALLI_DIR;
  BALL_OFF();
  IO_REGS.mcra &= ~BALL_MCRA;

  // set motor bits
  // set start value
  MTR0P_OFF();
  MTR0N_OFF();
  MTR1P_OFF();
  MTR1N_OFF();
  MTR2P_OFF();
  MTR2N_OFF();
  MTR3P_OFF();
  MTR3N_OFF();

  // set direction
  MTR0P_REG |= MTR0P_DIR;
  MTR0N_REG |= MTR0N_DIR;
  MTR1P_REG |= MTR1P_DIR;
  MTR1N_REG |= MTR1N_DIR;
  MTR2P_REG |= MTR2P_DIR;
  MTR2N_REG |= MTR2N_DIR;
  MTR3P_REG |= MTR3P_DIR;
  MTR3N_REG |= MTR3N_DIR;

  MTR3P_OFF();
  MTR2P_OFF();
  MTR1P_OFF();
  MTR0P_OFF();

  // set control bits
  IO_REGS.mcra &= ~MTR_MCRA;
  IO_REGS.mcrc &= ~MTR_MCRC;

  // setup the ADC conversions
  // continuous mode, cascaded on all channels
  adcparams.cascade = TRUE;
  adcparams.continuous = TRUE;
  initADC(&adcparams);

  adcset.nr_conversions = 6;
  adcset.reset = TRUE;
  adcset.channels[ADCRES_MTR0] = 3;
  adcset.channels[ADCRES_MTR1] = 2;
  adcset.channels[ADCRES_MTR2] = 1;
  adcset.channels[ADCRES_MTR3] = 0;
  adcset.channels[ADCRES_VBATT] = 6;
  adcset.channels[ADCRES_RSSI] = 7;

  // start the conversions
  ADCStartConversion(&adcset);
 
  return (TRUE);
}


/*
 * WaitForButton -
 *
 * function waits for a button press. Wait for press and release
 */
inline void WaitForButton(void)
{
  while (!GET_PB())
    ;
  while (GET_PB())
    ;
}


/*
 * KickDog -
 *
 * funciton to kick the watchdog. Inline for speed
 */
inline void KickDog(void)
{
  WD_REGS.wdkey = WD_KICKA;
  WD_REGS.wdkey = WD_KICKB;
}
	
/*
 * FlashLed -
 *
 * This function flashes the LED for d times with a delay of n
 */
void FlashLed(int d, int n)
{
  while (n-- > 0) {
    LED_ON();
    SimpleDelay(d);
    LED_OFF();
    SimpleDelay(d);
  }
}	

void FlashLedi(int led, int d, int n)
{
  ushort bit = LED1_BIT;
  if (led == 2)
    bit = LED2_BIT;
  else if (led == 3)
    bit = LED3_BIT;

  while (n-- > 0) {
    LEDI_ON(bit);
    SimpleDelay(d);
    LEDI_OFF(bit);
    SimpleDelay(d);
  }
}


/*
 * SimpleDelay -
 *
 * this is a simple delay function implemented with a nested loop. 
 */
void SimpleDelay(int d)
{
  int i;
	
  while (d-- > 0) {
    for (i = 0; i < 10; i++)
      ;
    KickDog();
  }
}

/*
 * ReadEncoder3 -
 *
 * This function reads the 16 bit value stored in teh external encoder
 * and returns it
 */
inline unsigned short ReadEncoder3(void)
{
  unsigned short rval;

  QDEC_SEL_LOW();
  QDEC_OE_LOW();
  rval = (port0 & 0xFF);
  rval <<= 8;
  QDEC_OE_HI();
  QDEC_SEL_HI();
  QDEC_OE_LOW();
  rval |= (port0 & 0xFF);
  QDEC_OE_HI();
  return (rval);
}


/*
 * SetMotor -
 *
 * this function sets teh motor pwm values. It determines
 * which PWM channel to set for each motor
 */
void SetMotor(int pwm1, int pwm2, int pwm3)
{
  /*
  if (pwm1 < 0) {
    pwm1 = MAX(pwm1, -MAX_PWM);
    SetDBPWM(MOTOR1_NEG, 0);
    SetDBPWM(MOTOR1_POS, MAX_PWM + pwm1);
  } else {
    pwm1 = MIN(pwm1, MAX_PWM);
    SetDBPWM(MOTOR1_NEG, pwm1);
    SetDBPWM(MOTOR1_POS, MAX_PWM);
  }
  if (pwm2 < 0) {
    pwm2 = MAX(pwm2, -MAX_PWM);
    SetDBPWM(MOTOR2_NEG, 0);
    SetDBPWM(MOTOR2_POS, MAX_PWM + pwm2);
  } else {
    pwm2 = MIN(pwm2, MAX_PWM);
    SetDBPWM(MOTOR2_NEG, pwm2);
    SetDBPWM(MOTOR2_POS, MAX_PWM);
  }
  
  if (pwm3 < 0) {
    pwm3 = MAX(pwm3, -MAX_PWM);
    SetDBPWM(MOTOR3_NEG, 0);
    SetDBPWM(MOTOR3_POS, MAX_PWM + pwm3);
  } else {
    pwm3 = MIN(pwm3, MAX_PWM);
    SetDBPWM(MOTOR3_NEG, pwm3);
    SetDBPWM(MOTOR3_POS, MAX_PWM);
  }
  */
}




void SetMotorNew(int pwm1, int pwm2, int pwm3, int pwm4)
{
  SetMotor0(pwm1);
  SetMotor1(pwm2);
  SetMotor2(pwm3);
  SetMotor3(pwm4);
}


/*
 * bad_trap -
 *
 * This funciton is the ISR for the bad_trap unitialised interrupt routine
 * it just loops forever doing nothing so that the watchdog will timeout
 */
void bad_trap(void)
{
  for (;;)
    LED_ON();
}

void SetMotor0(int pwm)
{
  if (pwm > 0) {
    MTR0N_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR0P_ON();
    SetDBPWM(MTR0_PWR, pwm);
  } else {
    MTR0P_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR0N_ON();
    SetDBPWM(MTR0_PWR, MAX_PWM + pwm);
  }
}


void SetMotor1(int pwm)
{
  if (pwm > 0) {
    MTR1N_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR1P_ON();
    SetDBPWM(MTR1_PWR, pwm);
  } else {
    MTR1P_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR1N_ON();
    SetDBPWM(MTR1_PWR, MAX_PWM + pwm);
  }
}


void SetMotor2(int pwm)
{
  if (pwm > 0) {
    MTR2N_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR2P_ON();
    SetDBPWM(MTR2_PWR, pwm);
  } else {
    MTR2P_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR2N_ON();
    SetDBPWM(MTR2_PWR, MAX_PWM + pwm);
  }
}


void SetMotor3(int pwm)
{
  if (pwm > 0) {
    MTR3N_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR3P_ON();
    SetDBPWM(MTR3_PWR, pwm);
  } else {
    MTR3P_OFF();
    fastDelay(MTR_DIR_CHANGE_DELAY);
    MTR3N_ON();
    SetDBPWM(MTR3_PWR, MAX_PWM + pwm);
  }
}

inline void fastDelay(int d)
{
  while (d-- > 0)
    ;
}

void memset(char *mem, int v, int count)
{
  while (count-- > 0)
    *mem++ = (v) & 0xFF;
}

void memcpy(char *dst, char *src, int count)
{
  while (count-- > 0)
    *dst++ = *src++;
}

