/*
 * TITLE:		Board.h
 *
 * DESCRIPTION:	This file contains the utility functions to run various parts of the LF2407 peripherals 
 *				specific to the board.
 *
 * WRITTEN BY:	Brett Browning
 *
 * REVISION HISTORY:
 * 25 May 2001		- Created (BB)
 *
 */
/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */ 
#ifndef _BOARD_H
#define _BOARD_H

#include "lf2407.h"

/**************************** TYPES & MACROS ********************************/

/* enable and disable interrupt macros */
#define EI()			asm("	CLRC	INTM;")
#define DI()			asm("	SETC	INTM;")

/* max pwm value for 19kHz period with 40MHz clock */
#define MAX_PWM			1024

/* crappy delay values */
#define FAST_DELAY		0x7FF
#define SLOW_DELAY		0x4FFF

/* Macro to turn on and off the LED */
#define LED_MCRB                0x01C0
#define LED1_DIR		0x0400
#define LED2_DIR		0x0800
#define LED3_DIR		0x1000
#define LED1_BIT		0x0004
#define LED2_BIT		0x0008
#define LED3_BIT		0x0010
#define LED1_ON()		IO_REGS.pcddr |= LED1_BIT
#define LED1_OFF()		IO_REGS.pcddr &= ~LED1_BIT
#define LED2_ON()		IO_REGS.pcddr |= LED2_BIT
#define LED2_OFF()		IO_REGS.pcddr &= ~LED2_BIT
#define LED3_ON()		IO_REGS.pcddr |= LED3_BIT
#define LED3_OFF()		IO_REGS.pcddr &= ~LED3_BIT
#define LED1(v)		        ((v) ? (LED1_ON()) : (LED1_OFF()))
#define LED2(v)		        ((v) ? (LED2_ON()) : (LED2_OFF()))
#define LED3(v)		        ((v) ? (LED3_ON()) : (LED3_OFF()))

#define LEDI_ON(bit)   IO_REGS.pcddr |= (bit)
#define LEDI_OFF(bit)  IO_REGS.pcddr &= ~(bit)

#define LEDI(bit, v)		((v) ? (LED_ON(bit)) : (LED_OFF(bit)))

#define LED_ON()    LED1_ON()
#define LED_OFF()   LED1_OFF()
#define LED(v)      LED1(v)

// motor driver extra bits
// MTR0P/MTR0N - PA5, PF1
// MTR1P/MTR1N - PWM1 (PA6), PWM2 (PA7)
// MTR2P/MTR2N - PWM5 (PB2), PWM6 (PB3)
// MTR3P/MTR3N - PF3, PF2

// MCRA - 
// PA5      - bit 5
// PA6/PWM1 - bit 6
// PA7/PWM2 - bit 7
// PB2/PWM5 - bit 10
// PB3/PWM6 - bit 11
// = 0x0CE0
#define MTR_MCRA    0x0CE0

// MCRC -
// PF1      - bit 9
// PF2      - bit 10
// PF3      - bit 11
// = 0x0E00
#define MTR_MCRC    0x0E00

// Motor 0P --------------------------
#define MTR0P_REG   IO_REGS.paddr
#define MTR0P_DIR   0x2000
#define MTR0P_BIT   0x0020

// are our level shifters inverted or not?
#ifdef INVMOTOR
#define MTR0P_ON()  IO_REGS.paddr |= MTR0P_BIT // hack - switch
#define MTR0P_OFF()   IO_REGS.paddr &= ~MTR0P_BIT
#else
#define MTR0P_OFF()  IO_REGS.paddr |= MTR0P_BIT
#define MTR0P_ON()   IO_REGS.paddr &= ~MTR0P_BIT
#endif

#define GET_MTR0P()  (!(MTR0P_REG & MTR0P_BIT))

// Motor 0N --------------------------
#define MTR0N_REG   IO_REGS.pfddr
#define MTR0N_DIR   0x0200
#define MTR0N_BIT   0x0002
#define GET_MTR0N()  (!(MTR0N_REG & MTR0N_BIT))

#ifdef INVMOTOR
#define MTR0N_OFF()  MTR0N_REG |= MTR0N_BIT // hack - switch
#define MTR0N_ON()  MTR0N_REG &= ~MTR0N_BIT
#else
#define MTR0N_ON()  MTR0N_REG |= MTR0N_BIT
#define MTR0N_OFF()  MTR0N_REG &= ~MTR0N_BIT
#endif


// Motor 1P --------------------------
#define MTR1P_REG   IO_REGS.paddr
#define MTR1P_DIR   0x4000
#define MTR1P_BIT   0x0040
#define GET_MTR1P()  (!(MTR1P_REG & MTR1P_BIT))

#ifdef INVMOTOR
#define MTR1P_ON()  MTR1P_REG |= MTR1P_BIT // hack - switch
#define MTR1P_OFF()  MTR1P_REG &= ~MTR1P_BIT
#else
#define MTR1P_OFF()  MTR1P_REG |= MTR1P_BIT
#define MTR1P_ON()  MTR1P_REG &= ~MTR1P_BIT
#endif


// Motor 1N --------------------------
#define MTR1N_REG   IO_REGS.paddr
#define MTR1N_DIR   0x8000
#define MTR1N_BIT   0x0080
#define GET_MTR1N()  (!(MTR1N_REG & MTR1N_BIT))

#ifdef INVMOTOR
#define MTR1N_OFF()  MTR1N_REG |= MTR1N_BIT // hack - switch
#define MTR1N_ON()  MTR1N_REG &= ~MTR1N_BIT
#else
#define MTR1N_ON()  MTR1N_REG |= MTR1N_BIT
#define MTR1N_OFF()  MTR1N_REG &= ~MTR1N_BIT
#endif


// Motor 2P --------------------------
#define MTR2P_REG   IO_REGS.pbddr
#define MTR2P_DIR   0x0400
#define MTR2P_BIT   0x0004
#define GET_MTR2P()  (!(MTR2P_REG & MTR2P_BIT))

#ifdef INVMOTOR
#define MTR2P_ON()  MTR2P_REG |= MTR2P_BIT // hack -- switch
#define MTR2P_OFF()  MTR2P_REG &= ~MTR2P_BIT
#else
#define MTR2P_OFF()  MTR2P_REG |= MTR2P_BIT
#define MTR2P_ON()  MTR2P_REG &= ~MTR2P_BIT
#endif


// Motor 2N --------------------------
#define MTR2N_REG   IO_REGS.pbddr
#define MTR2N_DIR   0x0800
#define MTR2N_BIT   0x0008
#define GET_MTR2N()  (!(MTR2N_REG & MTR2N_BIT))

#ifdef INVMOTOR
#define MTR2N_OFF()  MTR2N_REG |= MTR2N_BIT // hack - switch
#define MTR2N_ON()  MTR2N_REG &= ~MTR2N_BIT
#else
#define MTR2N_ON()  MTR2N_REG |= MTR2N_BIT
#define MTR2N_OFF()  MTR2N_REG &= ~MTR2N_BIT
#endif

// Motor 3P --------------------------
#define MTR3P_REG   IO_REGS.pfddr
#define MTR3P_DIR   0x0400
#define MTR3P_BIT   0x0004
#define GET_MTR3P()  (!(IO_REGS.pfddr & MTR3P_BIT))

#ifdef INVMOTOR
#define MTR3P_ON()  IO_REGS.pfddr |= MTR3P_BIT // hack -- switch
#define MTR3P_OFF()  IO_REGS.pfddr &= ~MTR3P_BIT
#else
#define MTR3P_OFF()  IO_REGS.pfddr |= MTR3P_BIT
#define MTR3P_ON()  IO_REGS.pfddr &= ~MTR3P_BIT
#endif


// Motor 3N --------------------------
#define MTR3N_REG   IO_REGS.pfddr
#define MTR3N_DIR   0x0800
#define MTR3N_BIT   0x0008
#define GET_MTR3N()  (!(MTR3N_REG & MTR3N_BIT))

#ifdef INVMOTOR
#define MTR3N_OFF()  IO_REGS.pfddr |= MTR3N_BIT // hack -- switch
#define MTR3N_ON()  IO_REGS.pfddr &= ~MTR3N_BIT
#else
#define MTR3N_ON()  IO_REGS.pfddr |= MTR3N_BIT
#define MTR3N_OFF()  IO_REGS.pfddr &= ~MTR3N_BIT
#endif



// motor channels
#define MTR0_PWR     7
#define MTR0_PWR2    (MTR0_PWR + 1)
#define MTR1_PWR     9
#define MTR1_PWR2    (MTR1_PWR + 1)
#define MTR2_PWR     3
#define MTR2_PWR2    (MTR2_PWR + 1)
#define MTR3_PWR     11
#define MTR3_PWR2    (MTR3_PWR + 1)

// channels used in bit fields
// EVA - channels 3,4, - 0x0018
// EVB - channels 7,8,9,10,11,12 - 0x
#define MTR_CHANNELS  (EVX_CH3 | EVX_CH4 | EVX_CH7 | EVX_CH8 \
                      | EVX_CH9 | EVX_CH10 | EVX_CH11 | EVX_CH12)


/* Macro to turn on and off the DRIBBLER */
#define DRIB_CTRL		0x1000
#define DRIB_DIR		0x1000
#define DRIB_BIT		0x0010
#define DRIB_OFF()		IO_REGS.pbddr |= DRIB_BIT
#define DRIB_ON()		IO_REGS.pbddr &= ~DRIB_BIT
#define DRIB(v)			((v) ? (DRIB_ON()) : (DRIB_OFF()))

/* bits for the Select pin for QDEC3 */
#define QDEC_MCRB	0x0100
#define QDEC_MCRC	0x2000
#define QDEC_SEL_DIR	0x0100
#define QDEC_SEL_BIT	0x0001
#define QDEC_OE_DIR	0x2000
#define QDEC_OE_BIT	0x0020
#define QDEC_SEL_HI()	IO_REGS.pdddr |= QDEC_SEL_BIT
#define QDEC_SEL_LOW()	IO_REGS.pdddr &= ~QDEC_SEL_BIT
#define QDEC_OE_HI()	IO_REGS.pfddr |= QDEC_OE_BIT
#define QDEC_OE_LOW()	IO_REGS.pfddr &= ~QDEC_OE_BIT

// Kicker bits -- kicker and KEN are inverted, but KEN is active low
#define KICK_MCRC   0x1000
#define KICK_DIR    0x4000
#define KICK_BIT    0x0040
#define KEN_DIR     0x1000
#define KEN_BIT     0x0010
#define KICK_ON()   IO_REGS.pfddr &= ~KICK_BIT
#define KICK_OFF()  IO_REGS.pfddr |= KICK_BIT
#define KEN_ON()   IO_REGS.pfddr |= KEN_BIT
#define KEN_OFF()    IO_REGS.pfddr &= ~KEN_BIT

// ball sensor bits -- both are inverted
#define BALL_MCRA   0x3000
#define BALLO_DIR   0x1000
#define BALLO_BIT   0x0010
#define BALLI_DIR   0x2000
#define BALLI_BIT   0x0020
#define BALL_ON()   IO_REGS.pbddr &= ~BALLO_BIT
#define BALL_OFF()  IO_REGS.pbddr |= BALLO_BIT
#define GET_BALL()  (IO_REGS.pbddr & BALLI_BIT)


/* settings for SARAM modes */
#define SARAM_OFF		0
#define SARAM_PROG		1
#define SARAM_DATA		2
#define SARAM_BOTH		3

/* BoardParam structure 
 * wd_en		- enable watchdog timer
 * wdper		- watchdog timeout in milliseconds
 * saram_mode	- SA RAM mode
 */
typedef struct tagBoardParam {
	ushort	wd_en, wdper;
	ushort	saram_mode;
} BoardParam;

/* MAcro to read from teh push button */
#define PB_MCRA     0x4000
#define PB_MCRB     0x0020
#define PB0_DIR     0x8000
#define PB0_BIT	    0x0080
#define PB1_DIR     0x2000
#define PB1_BIT	    0x0020
#define GET_PB0()   (!(IO_REGS.pbddr & PB0_BIT))
#define GET_PB1()   (!(IO_REGS.pcddr & PB1_BIT))
#define GET_PB() GET_PB0()

// ADC bits
#define ADCRES_VBATT    4
#define ADCRES_RSSI     5
#define ADCRES_MTR0     0
#define ADCRES_MTR1     1
#define ADCRES_MTR2     2
#define ADCRES_MTR3     3

#define ADC_CONV(v)     ((ushort) ((v) / 3.3 * 0x7FF))
#define GET_VBATT()     ADCReadResult(ADCRES_VBATT)
#define GET_RSSI()      ADCReadResult(ADCRES_RSSI)
#define GET_MTR0_CURR() ADCReadResult(ADCRES_MTR0)
#define GET_MTR1_CURR() ADCReadResult(ADCRES_MTR1)
#define GET_MTR2_CURR() ADCReadResult(ADCRES_MTR2)
#define GET_MTR3_CURR() ADCReadResult(ADCRES_MTR3)

	
/**************************** PROTOTYPES ***************************/

/*
 * InitBoard -
 *
 * This function intialises the various non-specific perihperals 
 * required for the smallsize 1.00 boards. It initializes the board
 * and system registers based upon the board parameters structure.
 * mem_ws, io_ws	- wait states for memory (program and data) and IO
 * wd_en, wdclk		- watch dog enable and clock setting (in ms)
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitBoard(BoardParam *bp);

/*
 * WaitForButton -
 *
 * function waits for a button press
 */
void WaitForButton(void);

/*
 * KickDog -
 *
 * funciton to kick the watchdog
 */
void KickDog(void);

/*
 * FlashLed -
 *
 * This function flashes the LED for d times with a delay of n
 */
void FlashLed(int d, int n);
void FlashLedi(int led, int d, int n);

/*
 * SimpleDelay -
 *
 * this is a simple delay function implemented with a nested loop. 
 */
void SimpleDelay(int d);


/*
 * ReadEncoder3 -
 *
 * This function reads the 16 bit value stored in teh external encoder
 * and returns it
 */
inline unsigned short ReadEncoder3(void);

/*
 * SetMotor -
 *
 * this function sets teh motor pwm values. It determines
 * which PWM channel to set for each motor
 */
void SetMotor(int pwm1, int pwm2, int pwm3);

// individual motor setting commands
void SetMotor0(int pwm);
void SetMotor1(int pwm);
void SetMotor2(int pwm);
void SetMotor3(int pwm);


/*
 * bad_trap -
 *
 * This funciton is the ISR for the bad_trap unitialised interrupt routine
 * it just loops forever doing nothing so that the watchdog will timeout
 */
void bad_trap(void);

void SetMemory(char *mem, int v, int count);
void MemCopy(char *dst, char *src, int count);

#endif /* _BOARD_H */
