/*
 * TITLE:		EVx.c
 *
 * DESCRIPTION:	This file encapsulates the TMS320LF2407 Event Manager Units of which there
 *				are two. CPU. The software here is based on the TMS320LF2407 documentation 
 *				and App notes -
 *				SPRU357		- DSP Controllers REference Guide : System and Peripherals
 *				
 *
 * WRITTEN BY:	Brett Browning
 *				Ravi Balasubarasaman
 *
 * REVISION HISTORY:
 * 10 Feb 2001		- Created (BB)
 *
 * TO DO -
 *
 */

/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#include "lf2407.h"
#include "evx.h"

/******************************* TYPES ****************************************/

/* Channels associated with each timer */
#define TIMER1_PWM_CHAN		(EVX_CH1 | EVX_CH2 | EVX_CH3 | EVX_CH4 | EVX_CH5 | EVX_CH6)
#define TIMER2_PWM_CHAN		(EVX_CH1 | EVX_CH2 | EVX_CH3 | EVX_CH4 | EVX_CH5 | EVX_CH6)
#define TIMER3_PWM_CHAN		(EVX_CH7 | EVX_CH8 | EVX_CH9 | EVX_CH10 | EVX_CH11 | EVX_CH12)
#define TIMER4_PWM_CHAN		(EVX_CH7 | EVX_CH8 | EVX_CH9 | EVX_CH10 | EVX_CH11 | EVX_CH12)

/*
 * Timer Control bits
 * TCON_EMSTOP		- stop on emulation suspend
 * TCON_EMPER		- stop at end of period on emulation suspend
 * TCON_IGNORE		- keep going regardless
 *
 * TCON_STOP		- Stop hold mode
 * TCON_CUPDWN		- Continuous Up/Down mode
 * TCON_CUP			- Continuous Up mode
 * TCON_DUPDWN		- Directional Up down mode
 *
 * TCON_TPS			- prescaler mask
 * TCON_TPSHIFT		- prescaler shift
 *
 * T2SWT1, T4SWT3	- use other TENABLE
 * TENABLE			- enable
 * TCLKS_INT		- internal clock source
 * TCLKS_EXT		- external clock source
 * TCLKS_QEP		- QEP circuit
 * TCLD_ZERO		- reload counter when it = 0 
 * TCLD_PER			- reload when = 0 or period value
 * TCLD_NOW			- reload now
 * TECMPR			- Enable Timer compare
 * SELT1PR, SELT3PR	- Period select for Timer 1 and 3
 */
#define EMSTOP			0x0000
#define EMPER			0x4000
#define IGNORE			0x8000
#define STOP			0x0000
#define CUPDWN			0x0800
#define CUP				0x1000
#define DUPDWN			0x1800
#define TPS				0x0700
#define TPSHIFT			8
#define T2SWT1			0x0080
#define T4SWT3			0x0080
#define TENABLE			0x0040
#define TCLKS_INT		0x0000
#define TCLKS_EXT		0x0010
#define TCLKS_QEP		0x0030
#define TCLD_ZERO		0x0000
#define TCLD_PER		0x0004
#define TCLD_NOW		0x0008
#define TECMPR			0x0002
#define SELT1PR			0x0001
#define SELT3PR			0x0001

#define T1CON_VAL		(IGNORE | CUP | TCLKS_INT | TCLD_PER | TECMPR)
#define TCON_DBPWM		(IGNORE | CUPDWN | TCLKS_INT | TCLD_ZERO | TECMPR)
#define TCON_UP			(IGNORE | CUP | TCLKS_INT | TCLD_ZERO | TECMPR)
#define TCON_QEP		(IGNORE | CUPDWN | TCLKS_QEP | TCLD_ZERO)

/*
 * GPTCON
 * TxSTAT_UP		- Counter is counting up bit
 * TxTOADC_yyy		- Conditions for astarting the ADC of timer x
 * TCOMPOE			- Compare output enable
 * TxPIN_yyy		- set the polarity of the GP timer x compare output
 */
#define T2STAT_UP		0x4000
#define T1STAT_UP		0x2000
#define T2TOADC_NONE	0x0000
#define T2TOADC_UFLW	0x0200
#define T2TOADC_PER		0x0400
#define T2TOADC_CMPI	0x0600
#define T1TOADC_NONE	0x0000
#define T1TOADC_UFLW	0x0080
#define T1TOADC_PER		0x0100
#define T1TOADC_CMPI	0x0180
#define TCOMPOE			0x0040
#define T2PIN_FLOW		0x0000
#define T2PIN_LOW		0x0004
#define T2PIN_HIGH		0x0008
#define T2PIN_FHIGH		0x000C
#define T1PIN_FLOW		0x0000
#define T1PIN_LOW		0x0001
#define T1PIN_HIGH		0x0002
#define T1PIN_FHIGH		0x0003

#define GPTCONA_VAL		(TCOMPOE | T1PIN_HIGH | T2PIN_HIGH)

/*
 * COMCON -
 * CENABLE		- Enable compare operation
 * CLD_yyy		- Compare register reload
 * SVENABLE		- Space vector PWM enable
 * ACTRLD_yyy	- Action control register reload condition
 * FCOMPOE		- Compare output enable on PDPINT
 */
#define CENABLE			0x8000
#define CLD_ZERO		0x0000
#define CLD_PER			0x2000
#define CLD_NOW			0x4000
#define SVENABLE		0x1000
#define ACTRLD_ZERO		0x0000
#define ACTRLD_PER		0x0400
#define ACTRLD_NOW		0x0800
#define FCOMPOE			0x0200

#define COMCON_VAL			(CLD_ZERO | ACTRLD_ZERO | FCOMPOE)

#define COMCON_DBPWM		(FCOMPOE)

/* ACTRA -
 * SVDIR_NEG		- space vecotr rotation
 */
#define SVRDIR_NEG			0x8000
#define CMPxACT_FLOW		0x0000
#define CMPxACT_LOW			0x0001
#define CMPxACT_HIGH		0x0002
#define CMPxACT_FHIGH		0x0003


/* Dead band control register */
#define DBT_SHIFT			8
#define DBT_MASK			0xF
#define DBT_ENABLE3			0x0080
#define DBT_ENABLE2			0x0040
#define DBT_ENABLE1			0x0020
#define DBT_PSMASK			0x07
#define DBT_PS_SHIFT		2

/* Cap COn register bits
 * RESET_CAPTURE		- macro to reset capture circuits
 * CAP_EN				- enable capture 1 and 2
 * QEP_EN				- enable QEP circuit
 * CAP3_EN				- enable capture 3
 * CAP3RSEL				- select GP Timer 1  (defulat is 0)
 * CAP12TSEL			- enable GP Timer 1  (defulat is 0)
 * CAP3TOADC			- set CAp 3 to trigger ADC conversion
 * CAP_xxx				- capture on edge type
 * CAPx_EDGE			- macro to set appropriate edge capture bits
 */
#define RESET_CAPTURE(m)	(EV##m##_REGS.capregs.capcon &= ~0x8000)
#define CAP_EN				0x2000
#define QEP_EN				0x6000
#define CAP3_EN				0x1000
#define CAP3TSEL			0x0400
#define CAP12TSEL			0x0200
#define CAP3TOADC			0x0100
#define CAP_NOEDGE			0x0000
#define CAP_RISING			0x0001
#define CAP_FALL			0x0002
#define CAP_BOTH			0x0003
#define CAP1_EDGE(t)		((t) << 6)
#define CAP2_EDGE(t)		((t) << 4)
#define CAP3_EDGE(t)		((t) << 2)

/* setting for QDEC */
#define CAPCON_QEP			(QEP_EN)

/*
 * InitEvx -
 *
 * This funciton initialises the EVx module
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitEvx(int a)
{
  int		i;
	
  /* enable the system clock to enable the module */
  SYS_REGS.scsr1 |= SCSR1_EVAEN | SCSR1_EVBEN;
	
  /* reset all the system parameters to 0 */
  EVA_REGS.gpregs.gptcon = 0;
  for (i = 0; i < 2; i++) {
    EVA_REGS.gpregs.timer[i].con = 0;
    EVA_REGS.gpregs.timer[i].cnt = 0;
    EVA_REGS.gpregs.timer[i].cmpr = 0;
    EVA_REGS.gpregs.timer[i].per = 0;
  }
  EVA_REGS.cmpregs.comcon = 0;
  for (i = 0; i < 3; i++)
    EVA_REGS.cmpregs.cmpr[i] = 0;
  EVA_REGS.cmpregs.dbtcon = EVA_REGS.cmpregs.actr = EVA_REGS.cmpregs.actr = 0;
  EVA_REGS.capregs.capcon = EVA_REGS.capregs.capfifox = 0;
  for (i = 0; i < 3; i++) {
    EVA_REGS.capregs.capfifo[i];
    EVA_REGS.capregs.capfbot[i];
  }
  
  /* clear the interrupt flags */
  EVA_REGS.intrregs.imra = EVA_REGS.intrregs.imrb = EVA_REGS.intrregs.imrc = 0;
  EVA_REGS.intrregs.ifra = EVA_REGS.intrregs.ifrb = EVA_REGS.intrregs.ifrc = 0;
  
  /* reset all the system parameters to 0 */
  EVB_REGS.gpregs.gptcon = 0;
  for (i = 0; i < 2; i++) {
    EVB_REGS.gpregs.timer[i].con = 0;
    EVB_REGS.gpregs.timer[i].cnt = 0;
    EVB_REGS.gpregs.timer[i].cmpr = 0;
    EVB_REGS.gpregs.timer[i].per = 0;
  }
  EVB_REGS.cmpregs.comcon = 0;
  for (i = 0; i < 3; i++)
    EVB_REGS.cmpregs.cmpr[i] = 0;
  EVB_REGS.cmpregs.dbtcon = EVB_REGS.cmpregs.actr = EVB_REGS.cmpregs.actr = 0;
  EVB_REGS.capregs.capcon = EVB_REGS.capregs.capfifox = 0;
  for (i = 0; i < 3; i++) {
    EVB_REGS.capregs.capfifo[i];
    EVB_REGS.capregs.capfbot[i];
  }
  
  /* clear the interrupt flags */
  EVB_REGS.intrregs.imra = EVB_REGS.intrregs.imrb = EVB_REGS.intrregs.imrc = 0;
  EVB_REGS.intrregs.ifra = EVB_REGS.intrregs.ifrb = EVB_REGS.intrregs.ifrc = 0;
}


/*
 * SetGPTimer -
 *
 * This function sets the GP timer of the appropriate EVx module
 * to the settings specified in the parameter list.
 *
 * RETURN VALUE: TRUEon success, FALSE on failure
 */
int SetGPTimer(int ch, int mode)
{
  ch--;
  if (ch < 2) {

    /* setup the timer */
    EVA_REGS.gpregs.timer[ch].con &= ~TENABLE;
    EVA_REGS.gpregs.timer[ch].con = TCON_UP;
    EVA_REGS.gpregs.timer[ch].per = mode;
    EVA_REGS.gpregs.timer[ch].cmpr = mode/2;
    EVA_REGS.cmpregs.comcon = 0;
    
    EVA_REGS.gpregs.timer[ch].con |= TENABLE;
  }
}

/*
 * InitPWM -
 *
 * This function initialises the channels to run as standard PWM
 * outputs. 
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitPWM(int chs, ushort period, ushort val)
{
  /* clear the period register and disable timer */
  EVA_REGS.gpregs.timer[0].con &= ~TENABLE;
  EVA_REGS.gpregs.timer[0].cnt = 0;
  EVA_REGS.gpregs.timer[0].per = period;
  EVA_REGS.gpregs.timer[0].cmpr = val;
  
  EVA_REGS.cmpregs.cmpr[0] = val;

  EVA_REGS.gpregs.timer[0].con = T1CON_VAL;
  EVA_REGS.cmpregs.comcon = COMCON_VAL;
  //	EVA_REGS.cmpregs.actr = ACTRA_VAL;

  EVA_REGS.cmpregs.comcon |= CENABLE;
  EVA_REGS.gpregs.timer[0].con |= TENABLE;

//	GPTCONA = GPTCONA_VAL;

}

/*
 * SetPWM -
 *
 * This funciton sets the PWM value of the given channel
 */
void SetPWM(int ch, ushort duty);

/*
 * InitDBPWM -
 *
 * This function initialises the deadband operation of the PWM channel.
 * Bit fields indicate which channels are to be used. By default the channels
 * must be paired and they use the base 0 timer (ie Timer 1 and 3).
 * pol describes the polarity for each channel.
 *
 * By default asymmetric PWM is used
 *
 * RETURN VALUE: RUEon success, FALSE on failure
 */
int InitDBPWM(ushort chs, ushort chpol, ushort per, ulong dband)
{
  ushort i, chpolhi, chpollo, dbten, palloc;
  ulong temp, presc;
	
  if (chs & TIMER1_PWM_CHAN) {
	
    /* setup the timer */
    EVA_REGS.gpregs.timer[0].con &= ~TENABLE;
    EVA_REGS.gpregs.timer[0].con = TCON_DBPWM;
    EVA_REGS.gpregs.timer[0].per = per;
    EVA_REGS.gpregs.timer[0].cmpr = 0;
    EVA_REGS.cmpregs.comcon = COMCON_DBPWM;

    /* default start PWM value is 0 */
    EVA_REGS.cmpregs.cmpr[0] = 0;
    EVA_REGS.cmpregs.cmpr[1] = 0;
    EVA_REGS.cmpregs.cmpr[2] = 0;
		
    /* 
     * setup the ACTR for each complementary channel 
     * - first channel active high, second active low 
     * Deadband enable and allocate the complementary pins
     */
    chpolhi = CMPxACT_HIGH;
    chpollo = CMPxACT_LOW;
    
    dbten = DBT_ENABLE1;
    palloc = MCRA_PWM1 | MCRA_PWM2;
    for (i = 1; i <= EVX_CH6; i <<= 2) {
      if (chs & i) {
	if (chpol & i) 
	  EVA_REGS.cmpregs.actr |= chpolhi;
	else
	  EVA_REGS.cmpregs.actr |= chpollo;
	chpollo <<= 2;
	chpolhi <<= 2;
	if (chpol & (i << 1))
	  EVA_REGS.cmpregs.actr |= chpolhi;
	else
	  EVA_REGS.cmpregs.actr |= chpollo;
	chpollo <<= 2;
	chpolhi <<= 2;

	EVA_REGS.cmpregs.dbtcon |= dbten;
	IO_REGS.mcra |= palloc;
      } else {
	chpollo <<= 4;
	chpolhi <<= 4;
      }			
      
      dbten <<= 1;
      palloc <<= 2;
    }
    
    /* work out smallest prescaler for dead band function */
    presc = 1;
    dbten = 0;
    while ((presc <= 32) && (((presc * DBT_MASK * 1000000L) / (CPU_FREQ / 1000L)) < dband)) {
      presc <<= 1;
      dbten++;
    }
    EVA_REGS.cmpregs.dbtcon |= (dbten << DBT_PS_SHIFT);
    
    /* work out value and store in registers */
    temp = ((dband * (CPU_FREQ / 1000L)) / (1000000L * presc));
    dbten = (ushort) temp;
    EVA_REGS.cmpregs.dbtcon |= ((dbten & DBT_MASK) << DBT_SHIFT);
    
    /* enable the timers to go */
    EVA_REGS.cmpregs.comcon |= CENABLE;
    EVA_REGS.gpregs.timer[0].con |= TENABLE;
  }
  
  /* Do the same for EVB */
  if (chs & TIMER3_PWM_CHAN) {
    
    /* setup the timer */
    EVB_REGS.gpregs.timer[0].con &= ~TENABLE;
    EVB_REGS.gpregs.timer[0].con = TCON_DBPWM;
    EVB_REGS.gpregs.timer[0].per = per;
    EVB_REGS.gpregs.timer[0].cmpr = 0;
    EVB_REGS.cmpregs.comcon = COMCON_DBPWM;
    
    /* default start PWM value is 0 */
    EVB_REGS.cmpregs.cmpr[0] = 0;
    EVB_REGS.cmpregs.cmpr[1] = 0;
    EVB_REGS.cmpregs.cmpr[2] = 0;
    
    /* 
     * setup the ACTR for each complementary channel 
     * - first channel active high, second active low 
     * Deadband enable and allocate the complementary pins
     */
    chpolhi = CMPxACT_HIGH;
    chpollo = CMPxACT_LOW;
    
    dbten = DBT_ENABLE1;
    palloc = MCRC_PWM7 | MCRC_PWM8;
    for (i = EVX_CH7; i <= EVX_CH12; i <<= 2) {
      if (chs & i) {
	if (chpol & i) 
	  EVB_REGS.cmpregs.actr |= chpolhi;
	else
	  EVB_REGS.cmpregs.actr |= chpollo;
	chpollo <<= 2;
	chpolhi <<= 2;
	if (chpol & (i << 1))
	  EVB_REGS.cmpregs.actr |= chpolhi;
	else
	  EVB_REGS.cmpregs.actr |= chpollo;
	chpollo <<= 2;
	chpolhi <<= 2;
	
	EVB_REGS.cmpregs.dbtcon |= dbten;
	IO_REGS.mcrc |= palloc;
      } else {
	chpollo <<= 4;
	chpolhi <<= 4;
			}			
      
      dbten <<= 1;
      palloc <<= 2;
    }
    
    /* work out smallest prescaler for dead band function */
    presc = 1;
    dbten = 0;
    while ((presc <= 32) && (((presc * DBT_MASK * 1000000L) / (CPU_FREQ / 1000L)) < dband)) {
      presc <<= 1;
      dbten++;
    }
    EVB_REGS.cmpregs.dbtcon |= (dbten << DBT_PS_SHIFT);
    
    /* work out value and store in registers */
    temp = ((dband * (CPU_FREQ / 1000L)) / (1000000L * presc));
    dbten = (ushort) temp;
    EVB_REGS.cmpregs.dbtcon |= ((dbten & DBT_MASK) << DBT_SHIFT);
    
    /* enable the timers to go */
    EVB_REGS.cmpregs.comcon |= CENABLE;
    EVB_REGS.gpregs.timer[0].con |= TENABLE;
  }
}


/*
 * SetDBPWM -
 *
 * This function sets the dead band duty cycle 
 */
inline void SetDBPWM(ushort ch, ushort duty)
{
  /* make zero based */
  ch--;
  if (ch < 6) {
    ch >>= 1;
    EVA_REGS.cmpregs.cmpr[ch] = duty;
  } else {
    ch -= 6;
    ch >>= 1;
    EVB_REGS.cmpregs.cmpr[ch] = duty;
  }
}

/*
 * getDBPWM -
 *
 * This function returns the dead band duty cycle 
 */
inline ushort getDBPWM(ushort ch)
{
  /* make zero based */
  ch--;
  if (ch < 6) {
    ch >>= 1;
    return (EVA_REGS.cmpregs.cmpr[ch]);
  } else {
    ch -= 6;
    ch >>= 1;
    return (EVB_REGS.cmpregs.cmpr[ch]);
  }
}


/*
 * InitQDEC -
 *
 * This function initialises the QDEC channels on the appropriate EV module.
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int InitQDEC(ushort ch)
{
  if (ch == 1) {
    EVA_REGS.gpregs.timer[1].con &= ~TENABLE;
    EVA_REGS.gpregs.timer[1].con = TCON_QEP;
    EVA_REGS.gpregs.timer[1].per = 0xFFFF;
    EVA_REGS.gpregs.timer[1].cmpr = 0;
    
    EVA_REGS.capregs.capcon = CAPCON_QEP;
    EVA_REGS.gpregs.timer[1].con |= TENABLE;
  } else {
    EVB_REGS.gpregs.timer[1].con &= ~TENABLE;
    EVB_REGS.gpregs.timer[1].con = TCON_QEP;
    EVB_REGS.gpregs.timer[1].per = 0xFFFF;
    EVB_REGS.gpregs.timer[1].cmpr = 0;
    
    EVB_REGS.capregs.capcon = CAPCON_QEP;
    EVB_REGS.gpregs.timer[1].con |= TENABLE;
  }
}


/*
 * ReadQDEC -
 *
 * This function reads the QDEC value from the appropriate count register
 *
 * RETURN VALUE: The QDEC count as a 16bit entity.
 */
inline ushort ReadQDEC(ushort ch)
{
  if (ch == 1)
    return (EVA_REGS.gpregs.timer[1].cnt);
  else
    return (EVB_REGS.gpregs.timer[1].cnt);
}

