/*
 * TITLE:  LF2407.h
 *
 * DESCRIPTION: This file encapsulates the memory architecture and register
 *    structure for the TMS320LF2407 CPU. All board specific structures
 *    are located in board.h. The register addresses defined here are based
 *    on the TMS320LF2407 documentation as -
 *    SPRU357  - DSP Controllers REference Guide : System and Peripherals
 *    SPRU160C - DSP Controllers REference Guide : CPU and Instruction Set
 *    SPRS094E - TMS320LF2407 Data Sheet
 *
 *    All are available from www.ti.com
 *
 * WRITTEN BY: Brett Browning
 *    Ravi Balasubarasaman
 *
 * REVISION HISTORY:
 * 10 Feb 2001  - Created (BB)
 *
 */

/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */

#ifndef _LF2407_H
#define _LF2407_H

/**************************** LF2407 Data Types ********************/
#ifndef FALSE
#define FALSE  0
#define TRUE  1
#endif

#ifndef PI
#define PI     3.14159
#endif

/* bound the int to max and min values */
#ifndef BOUND
#define BOUND(x, a, b)   ((x) > (a) ? (a) : ((x) < (b) ? (b) : (x)))
#endif

#ifndef MAX
#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
#endif

#ifndef ABS
#define ABS(a)   (((a) >= 0) ? (a) : -(a))
#endif
#ifndef SIGN
#define SIGN(a)   (((a) >= 0) ? 1 : -1)
#endif

typedef unsigned char  uchar;
typedef unsigned short  ushort;
typedef unsigned long  ulong;


/**************************** MISC BOARD settings ******************/
#define OSC_FREQ   10000000L //7372800L
#define CPU_FREQ   (OSC_FREQ * 4)

/* wait states */
#define MEM_WS    1
#define IO_WS    7

/**************************** Misc. registers **********************/
#define WSGR  (*((volatile ushort *) 0x0FFFF))
#define FCMR  (*((volatile ushort *) 0x0FF0F))

/* Wait state control bits */
#define WSGR_BUS_OFF  0
#define WSGR_DBUS   0x0400
#define WSGR_PBUS   0x0600

#define WSGR_SWS_MASK  0x7
#define WSGR_PSWS_SHIFT  0
#define WSGR_DSWS_SHIFT  3
#define WSGR_ISWS_SHIFT  6

/* macro to set teh wait statte bits */
#define WSGR_SWS_BITS(iw, dw, pw) ( (((iw) & WSGR_SWS_MASK) << 6) | \
     (((dw) & WSGR_SWS_MASK) << 3) | \
     ((pw) & WSGR_SWS_MASK))

/**************************** System Registers *********************/

typedef struct {
  ushort  pirqr[3];
  ushort  rsvd0;
  ushort  piakr[3];
  ushort  rsvd1;
  ushort  scsr1;
  ushort  scsr2;
  ushort  rsvd2[2];
  ushort  dinr;
  ushort  rsvd3;
  ushort  pivr;
} SystemRegs;

#define SYS_REGS  (*((volatile SystemRegs *) 0x07010))

/* PIRQ and PIACK bits */
#define IRQ_PDPINTA  0x0001
#define IRQ_ADCINT_H 0x0002
#define IRQ_XINT1_H  0x0004
#define IRQ_XINT2_H  0x0008
#define IRQ_SPIINT_H 0x0010
#define IRQ_RXINT_H  0x0020
#define IRQ_TXINT_H  0x0040
#define IRQ_CANMBINT_H 0x0080
#define IRQ_CANERINT_H 0x0100
#define IRQ_CMP1INT  0x0200
#define IRQ_CMP2INT  0x0400
#define IRQ_CMP3INT  0x0800
#define PIRQ_T1PINT  0x1000
#define IRQ_T1CINT  0x2000
#define IRQ_T1UFINT  0x4000
#define IRQ_T1OFINT  0x8000

#define IRQ_T2PINT  0x0001
#define IRQ_T2CINT  0x0002
#define IRQ_T2UFINT  0x0004
#define IRQ_T2OFINT  0x0008
#define IRQ_CAP1INT  0x0010
#define IRQ_CAP2INT  0x0020
#define IRQ_CAP3INT  0x0040
#define IRQ_SPIINT_L 0x0080
#define IRQ_RXINT_L  0x0100
#define IRQ_TXINT_L  0x0200
#define IRQ_CANMBINT_L 0x0400
#define IRQ_CANERINT_L 0x0800
#define IRQ_ADCINT_L 0x1000
#define IRQ_XINT1_L  0x2000
#define IRQ_XINT2_L  0x4000

#define IRQ_PDPINTB  0x0001
#define IRQ_CMP4INT  0x0002
#define IRQ_CMP5INT  0x0004
#define IRQ_CMP6INT  0x0008
#define IRQ_T3PINT  0x0010
#define IRQ_T3CINT  0x0020
#define IRQ_T3UFINT  0x0040
#define IRQ_T3OFINT  0x0080
#define IRQ_T4PINT  0x0100
#define IRQ_T4CINT  0x0200
#define IRQ_T4UFINT  0x0400
#define IRQ_T4OFINT  0x0800
#define IRQ_CAP4INT  0x1000
#define IRQ_CAP5INT  0x2000
#define IRQ_CAP6INT  0x4000

/* SCSR1_bits -
 * OSC_FAIL  - oscillator fail flag
 * CLKO_CPU  - clock out pin has CPU clock
 * CLKO_WD  - clock out pin has Watch dog clock
 * LPM0   - low power mode 0
 * LPM1   - low power mode 1
 * HALT   - low power mode Halt
 * RST_OSCF  - reset on oscillator fail
 */
#define SCSR1_OSC_FAIL 0x8000
#define SCSR1_CLKO_CPU 0x0000
#define SCSR1_CLKO_WD 0x4000
#define SCSR1_LPM0  0x0000
#define SCSR1_LPM1  0x1000
#define SCSR1_HALT  0x2000
#define SCSR1_RST_OSCF 0x0100

/* peripheral clock enables */
#define SCSR1_ADCEN  0x0080
#define SCSR1_SCIEN  0x0040
#define SCSR1_SPIEN  0x0020
#define SCSR1_CANEN  0x0010
#define SCSR1_EVBEN  0x0008
#define SCSR1_EVAEN  0x0004
#define SCSR1_ILLADDR 0x0001

/* PLL settings */
#define SCSR1_PLL_4  0x0000
#define SCSR1_PLL_2  0x0200
#define SCSR1_PLL_1_33 0x0400
#define SCSR1_PLL_1  0x0600
#define SCSR1_PLL_0_8 0x0800
#define SCSR1_PLL_0_66 0x0A00
#define SCSR1_PLL_0_57 0x0C00
#define SCSR1_PLL_0_5 0x0E00

/* SCSC2 bits
 * WDOVRIDE  - override Watch dog
 * XMIF_HIZ  - xmif to hi impedence
 * XM
 */
#define SCSR2_WDOVRIDE  0x0020
#define SCSR2_XMIF_HIZ  0x0010
#define SCSR2_XMIF_NORM  0x0000
#define SCSR2_BOOT_EN  0x0008
#define SCSR2_MP   0x0004
#define SCSR2_SARAM_OFF  0x0000
#define SCSR2_SARAM_PROG 0x0001
#define SCSR2_SARAM_DATA 0x0002
#define SCSR2_SARAM_BOTH 0x0003

/* macro to tell if in MP or MC mode - true if in Microprocessor mode (ie not flash) */
#define IS_MICRO_MODE()  (SYS_REGS.scsr2 & SCSR2_MP)

/* macro to tell if booted up via BOOTEN or not */
#define IS_BOOT_EN()  (!(SYS_REGS.scsr2 & SCSR2_BOOT_EN))

/**************************** WAtchdog Registers ************************/

typedef struct {
  ushort  wdcntr;
  ushort  rsvd1;
  ushort  wdkey;
  ushort  rsvd2[3];
  ushort  wdcr;
} WatchDogRegs;

#define WD_REGS    (*((volatile WatchDogRegs *) 0x7023))

/* Watchdog control bits */
#define WDCHK  0x0028
#define WDDIS  0x0040
#define WD_KICKA 0xAA
#define WD_KICKB 0x55
#define WD_PS_BITS 0x0007

/**************************** SPI Registers ************************/

typedef struct {
  ushort spiccr;
  ushort spictrl;
  ushort spists;
  ushort rsvd0;
  ushort spibrr;
  ushort rsvd1;
  ushort spirxemu;
  ushort spirxbuf;
  ushort spitxbuf;
  ushort spidat;
  ushort rsvd2[5];
  ushort spipri;
} SPIRegs;

#define SPI_REGS  (*((volatile SPIRegs *) 0x7040))

/**************************** SCI Registers ************************/

typedef struct {
  ushort sciccr;
  ushort scictl1;
  ushort scihbaud;
  ushort scilbaud;
  ushort scictl2;
  ushort scirxst;
  ushort scirxemu;
  ushort scirxbuf;
  ushort rsvd0;
  ushort scitxbuf;
  ushort rsvd1[5];
  ushort scipri;
} SCIRegs;

#define SCI_REGS  (*((volatile SCIRegs *) 0x7050))

/**************************** INTR Registers ************************/

typedef struct {
  ushort xint1cr;
  ushort xint2cr;
} XINTRegs;

#define XINT_REGS  (*((volatile XINTRegs *) 0x7070))

typedef struct {
  ushort imr;
  ushort greg;
  ushort ifr;
} IntrRegs;

#define INTR_REGS  (*((volatile IntrRegs *) 0x00004))

/* IMR bits */
#define INT1_MASK  0x01
#define INT2_MASK  0x02
#define INT3_MASK  0x04
#define INT4_MASK  0x08
#define INT5_MASK  0x10
#define INT6_MASK  0x20

/**************************** IO Registers ************************/

typedef struct {
  ushort mcra, rsvd0;
  ushort mcrb, rsvd1;
  ushort mcrc;
  ushort peddr;
  ushort pfddr, rsvd2;
  ushort paddr, rsvd3;
  ushort pbddr, rsvd4;
  ushort pcddr, rsvd5;
  ushort pdddr;
} IORegs;

#define IO_REGS   (*((volatile IORegs *) 0x7090))

/* MCRA bits to set port A and B to non IO function */
#define MCRA_SCITXD   0x0001
#define MCRA_SCIRXD   0x0002
#define MCRA_XINT1   0x0004
#define MCRA_QEP1   0x0008
#define MCRA_QEP2   0x0010
#define MCRA_CAP3   0x0020
#define MCRA_PWM1   0x0040
#define MCRA_PWM2   0x0080
#define MCRA_PWM3   0x0100
#define MCRA_PWM4   0x0200
#define MCRA_PWM5   0x0400
#define MCRA_PWM6   0x0800
#define MCRA_T1PWM   0x1000
#define MCRA_T2PWM   0x2000
#define MCRA_TDIRA   0x4000
#define MCRA_TCLKINA  0x8000

/* MCRB bits to set port A and B to non IO function */
#define MCRB_WRITE   0x0001
#define MCRB_BIO   0x0002
#define MCRB_SPISIMO  0x0004
#define MCRB_SPISOMI  0x0008
#define MCRB_SPISCLK  0x0010
#define MCRB_SPISTE   0x0020
#define MCRB_CANTX   0x0040
#define MCRB_CANRX   0x0080
#define MCRB_XINT2   0x0100
#define MCRB_EMU0   0x0200
#define MCRB_EMU1   0x0400
#define MCRB_TCK   0x0800
#define MCRB_TDI   0x1000
#define MCRB_TDO   0x2000
#define MCRB_TMS   0x4000
#define MCRB_TMS2   0x8000

/* MCRC bits to set port A and B to non IO function */
#define MCRC_CLKOUT   0x0001
#define MCRC_PWM7   0x0002
#define MCRC_PWM8   0x0004
#define MCRC_PWM9   0x0008
#define MCRC_PWM10   0x0010
#define MCRC_PWM11   0x0020
#define MCRC_PWM12   0x0040
#define MCRC_QEP3   0x0080
#define MCRC_QEP4   0x0100
#define MCRC_CAP6   0x0200
#define MCRC_T3PWM   0x0400
#define MCRC_T4PWM   0x0800
#define MCRC_TDIRB   0x1000
#define MCRC_TCLKINB  0x2000

/* settings for output and hi value */
#define PxDDR_OUT   0x0001
#define PxDDR_HI   0x0001

/**************************** ADC Registers ************************/

#define NR_ADC_RESULTS 16
typedef struct {
  ushort adcctrl[2];
  ushort maxconv;
  ushort chselseq[4];
  ushort autoseqsr;
  ushort result[NR_ADC_RESULTS];
  ushort calibration; // reserved in LF2407A
} ADCRegs;

#define ADC_REGS  (*((volatile ADCRegs *) 0x70A0))

// control register 1 settings
#define ADCCTRL_RESET        0x4000
#define ADCCTRL_RUNSTOP      0x0000
#define ADCCTRL_RUNCOMPLETE  0x2000
#define ADCCTRL_RUNFREE      0x1000
#define ADCCTRL_PS_MASK      0x0F00
#define ADCCTRL_PS_SHIFT     8
#define ADCCTRL_CPS          0x0080
#define ADCCTRL_CONT         0x0040
#define ADCCTRL_IPRI_LOW     0x0020
#define ADCCTRL_IPRI_HI      0
#define ADCCTRL_CASCADE      0x0010

// control register 2
#define ADCCTRL_EVBSOC      0x8000
#define ADC_SEQ1_RST        0x4000
#define ADC_SEQ1_SOC        0x2000
#define ADC_SEQ1_BUSY       0x1000
#define ADC_SEQ1_IEN1       0x0400
#define ADC_SEQ1_IEN2       0x0800
#define ADC_SEQ1_IFLAG      0x0200
#define ADC_SEQ1_EVA        0x0100
#define ADC_SEQ1_EXT        0x0080
#define ADC_SEQ2_RST        0x0040
#define ADC_SEQ2_SOC        0x0020
#define ADC_SEQ2_BUSY       0x0010
#define ADC_SEQ2_IEN1       0x0004
#define ADC_SEQ2_IEN2       0x0008
#define ADC_SEQ2_IFLAG      0x0002
#define ADC_SEQ2_EVB        0x0001

//MAXCONV bit settings
#define MAXCONV_MASK        0x07
#define MAXCONV_SEQ1_MASK   0x07
#define MAXCONV_SEQ2_MASK   0x70
#define MAXCONV_SEQ2_SHIFT  4
#define MAXCONV_SEQ_MASK    0x0F

// Auto sequence status register
#define AUTOSEQ1_MASK   0x000F
#define AUTOSEQ2_MASK   0x0070
#define AUTOSEQ2_SHIFT  4
#define AUTOSEQ_MASK    0x0F00
#define AUTOSEQ_SHIFT   8

// channel select sequencing bits
#define ADC_CHSEL_MASK     0xF
#define ADC_CHSEL_SHIFT    4
#define ADC_CHSEL_CHPERREG 4

// results are left justified
#define ADC_RESULT_SHIFT   5
#define ADC_RESULT_MASK    0x3FF


/**************************** CANR Registers ************************/

typedef struct {
  ushort mder;
  ushort tcr;
  ushort rcr;
  ushort mcr;
  ushort bcr2;
  ushort bcr1;
  ushort esr;
  ushort gsr;
  ushort cec;
  ushort canifr;
  ushort canimr;
  ulong lam0;
  ulong lam1;
} CANRegs;

typedef struct {
  ushort msgidl;
  ushort msgidh;
  ushort msgctrl;
  ushort rsvd0;
  ushort mbxa;
  ushort mbxb;
  ushort mbxc;
  ushort mbxd;
} CANMsg;

typedef struct {
  CANMsg  canmsg[6];
} CANMsgs;

#define CAN_REGS  (*((volatile CANRegs *) 0x7100))
#define CAN_MSGS  (*((volatile CANMsgs *) 0x7200))

/**************************** EVXR Registers ************************/

typedef struct {
  ushort cnt;
  ushort cmpr;
  ushort per;
  ushort con;
} EVxTimer;

typedef struct {
  ushort  gptcon;
  EVxTimer timer[2];
  ushort  rsvd0[8];
} EVxGPRegs;

typedef struct {
  ushort  comcon;
  ushort  rsvd0;
  ushort  actr;
  ushort  rsvd1;
  ushort  dbtcon;
  ushort  rsvd2;
  ushort  cmpr[3];
  ushort  rsvd3[6];
} EVxCmpRegs;

typedef struct {
  ushort  capcon;
  ushort  rsvd0;
  ushort  capfifox;
  ushort  capfifo[3];
  ushort  rsvd1;
  ushort  capfbot[3];
  ushort  rsvd2[2];
} EVxCapRegs;
 
typedef struct {
  ushort  imra;
  ushort  imrb;
  ushort  imrc;
  ushort  ifra;
  ushort  ifrb;
  ushort  ifrc;
} EVxIntrRegs;

typedef struct {
  EVxGPRegs  gpregs;
  EVxCmpRegs  cmpregs;
  EVxCapRegs  capregs;
  EVxIntrRegs  intrregs;
} EVxRegs;

#define EVA_BASE  0x7400
#define EVB_BASE  0x7500
#define EVA_REGS  (*((volatile EVxRegs *) EVA_BASE))
#define EVB_REGS  (*((volatile EVxRegs *) EVB_BASE))


/**************************** FlashR Registers ************************/

typedef struct {
  ushort  pmpc;
  ushort  ctrl;
  ushort  waddr;
  ushort  wdata;
  ushort  tcr;
  ushort  enab;
  ushort  sect;
} FlashRegs;

#define FLASH_REGS (*((volatile FlashRegs *) 0x0))



#endif /* _LF2407_ */
