/*
 * TITLE:		SCI.c
 *
 * DESCRIPTION:	This file encapsulates the SCI module on the TMS320LF2407 DSP. 
 *				The code is based on the TMS320LF2407 documentation and App notes as -
 *				SPRU357		- DSP Controllers REference Guide : System and Peripherals
 *
 * WRITTEN BY:	Brett Browning
 *
 * REVISION HISTORY:
 * 10 Feb 2001		- Created (BB)
 * 6 April 2001		- Made workable. Code now works for Tx and Rx and variable baud rates. It doesn't have interrupt support yet. (BB)
 *
 * TO DO:
 * - Add interrupt support
 * - Add proper status feedback function
 * - Add send packet function
 */

/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#include "sci.h"
#include "lf2407.h"

/**************************** TYPES ********************************/

/* SCICCR Registers -
 *
 */
#define STOP_BIT_2		0x80
#define ODD_PARITY		0x00
#define EVEN_PARITY		0x40
#define PARITY_EN		0x20
#define LOOPBACK		0x10
#define ADDR_MODE		0x08
#define SCHR_MASK		0x07

/* SCTL1 params */
#define RXENA			0x01
#define TXENA			0x02
#define SLEEP			0x04
#define TXWAKE			0x08
#define SW_RESET		0x20
#define RXERR_IEN		0x40

/* SCTL2 parameters */
#define TXREADY			0x80
#define TXEMPTY			0x40
#define TX_IEN			0x01
#define RX_IEN			0x02

/* reciever status register bits */
#define RXERR			0x80
#define RXREADY			0x40
#define BRKDT			0x20
#define FE				0x10
#define OE				0x08
#define PE				0x04
#define RXWAKE			0x02

/* priority settings */
#define TX_PRIORITY		0x40
#define RX_PRIORITY		0x20
#define SCI_SOFT		0x10
#define SCI_FREE		0x08

/**************************** PROTOTYPES ***************************/

/**************************** CODE *********************************/


/*
 * InitSCI -
 *
 * This funciton initialises the SCI port on the LF2407 to the appropriate
 * settings 
 *
 * RETURN VALUE: TRUE on success, FALSE on faiolure
 */
int InitSCI(SCIParam *sciparam)
{
	ulong	baud;
	ushort	val;
	
	/* enable the SCI clock to enable the module */
	SYS_REGS.scsr1 |= SCSR1_SCIEN;

	/* set the control mode and data format */
	SCI_REGS.sciccr = 0;
	if (sciparam->nr_stop == 2)
		SCI_REGS.sciccr |= STOP_BIT_2;
	if (sciparam->parity == SCI_EVEN_PARITY)
		SCI_REGS.sciccr |= EVEN_PARITY | PARITY_EN;
	else if (sciparam->parity == SCI_ODD_PARITY)
		SCI_REGS.sciccr |= ODD_PARITY | PARITY_EN;
	SCI_REGS.sciccr |= (sciparam->nr_chars - 1) & SCHR_MASK;
	
	/* set up the control register and allocate the pins */
	SCI_REGS.scictl1 = 0;
	if (sciparam->ctrl & SCI_TXEN) {
		SCI_REGS.scictl1 |= TXENA;
		IO_REGS.mcra |= MCRA_SCITXD;
	}
	if (sciparam->ctrl & SCI_RXEN) {
		SCI_REGS.scictl1 |= RXENA;
		IO_REGS.mcra |= MCRA_SCIRXD;
	}

	/* calculate the baud rate give the clock frequency */
	baud = CPU_FREQ;
	baud /= ((ulong) sciparam->baud << 3);
	baud--;
	val = (ushort) baud; 
	SCI_REGS.scihbaud = val & 0xFF00;
	SCI_REGS.scilbaud = val & 0x00FF;
	
	/* intialise the interrupts */
	SCI_REGS.scictl2 = 0;
	if (sciparam->ctrl & SCI_TX_IEN)
		SCI_REGS.scictl2 |= TX_IEN;
	if (sciparam->ctrl & SCI_RX_IEN)
		SCI_REGS.scictl2 |= RX_IEN;
		
	SCI_REGS.scipri = 0;
	if (sciparam->ctrl & SCI_TX_HIPRIORITY)
		SCI_REGS.scipri |= TX_PRIORITY;
	if (sciparam->ctrl & SCI_RX_HIPRIORITY)
		SCI_REGS.scipri |= RX_PRIORITY;

	/* set the emulation not to affect the SCI */
	SCI_REGS.scipri | SCI_FREE;
	
	/* let her rip!!! */
	SCI_REGS.scictl1 |= SW_RESET;
}


/*
 * SCISend
 *
 * This function sends a single byte through the serial port 
 *
 * RETURN VALUE: TRUE on success, FALSE on failure
 */
int SCISend(ushort data)
{
	while (!(SCI_REGS.scictl2 & TXREADY))
		;
	SCI_REGS.scitxbuf = data;
}


/*
 * SCISendPacket -
 *
 * This function sends a packet out the serial port
 */
void SCISendPacket(ushort *data, ushort len);

/*
 * SCISendString -
 *
 * This function sends a string out the serial port
 */
void SCISendString(char *data)
{
	while (*data) {
		if (*data == '\n')
			SCISend((ushort) '\r');
		SCISend((ushort) *data++);
	}
}


/*
 * SCISendInt -
 *
 * This function sends an int value in hex format to the serial
 * port, which must be initialised
 * Hexchar is a bool that defines if it should send th e'0x' characthers at teh start
 */
void SCISendInt(ushort d, ushort hexchar)
{
	char	str[5];
	int		i;
	
	str[4] = 0;
	for (i = 3; i >= 0; i--) {
		str[i] = (d & 0xF);
		if (str[i] > 9)
			str[i] += 'A' - 0xA;
		else
			str[i] += '0';
		d >>= 4;
	}
	if (hexchar)
		SCISendString("0x");
	SCISendString(str);
}		

/*
 * SCIRecv -
 *
 * This function polls the serial port until new data has arrived
 */
ushort SCIRecv(void)
{
	while (!(SCI_REGS.scirxst & RXREADY))
		;
	return (SCI_REGS.scirxbuf);
}


/*
 * SCIStatus -
 *
 * This funciton returns a bit field indicating teh status of the serial
 * device.
 *
 * rETURN VALUE: a bit field indicating the status
 */
ushort SCIStatus(void);


