/*
 * TITLE:		drive.c
 *
 * DESCRIPTION:	This file tests the motor drivers based onthe received RF packet command
 *
 * WRITTEN BY:	Brett Browning
 *
 * REVISION HISTORY:
 * 5 April 2001		- Created (BB)
 *
 */
/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */

#include "lf2407.h"
#include "evx.h"
#include "sci.h"
#include "board.h"
#include "servo.h"
#include "adc.h"


/**************************** TYPES ********************************/


// kicker charge enable flag
// Packet2 Flags
#define RF_REVERSE_DX      0x01
#define RF_REVERSE_DY      0x02
#define RF_REVERSE_DA      0x04
#define RF_HALT            0x08
#define RF_KICK	           0x10
#define RF_DRIBBLE         0x20 // deprecated
#define RF_REVERSE_DRIBBLE 0x20 
#define RF_USE_CURRENT     0x40
#define RF_KICKER_CHARGE   0x80
#define RF_ALL             0xFF

#define RF_PID_BITS        0x07
#define RF_DRIBBLE_SHIFT   4
#define RF_DRIBBLE_MASK    0x0F

#define RF_PAD_BYTE_HI 0xFF
#define RF_PAD_BYTE_LO 0x00

/* comm packet for receiving our crappy information */
typedef struct tag_CommPacket {
  ushort id;       // from identifier[]
  ushort dx,dy;    // velocities in cm/sec
  ushort da;       // angular velocity in 1/16 rad/sec
  ushort flags;    // flag bits
  ushort drib_cnt; // 4 bits dribble, 4 bits seq number
  ushort checksumlo; // checksum for 5 data fields
  ushort checksumhi;
} CommPacket;

enum PacketStage {PADDING1 = 0, PADDING2, ROBOTID, MAINPACKET};

typedef struct tag_Command {
  int pid;
  //  RobotVel rvel;
  int rvx, rvy, rva;
  int drib;

  int halt;
  int use_current_sensing;
  int kick;
  int kcharge;
} Command;

/* baud rate for comms */
#define BAUD_RATE			38400 //19200

/* Macro to reverse */
#define RVS_FLAG(v, f, fbit)		(((f) & (fbit))? -(v) : (v))

/* timeout just to keep sanity - in characters received */
#define TIMEOUT				10

#ifdef RID0
#define ROBOT_ID	0
#endif
#ifdef RID1
#define ROBOT_ID	1
#endif
#ifdef RID2
#define ROBOT_ID	2
#endif
#ifdef RID3
#define ROBOT_ID	3
#endif
#ifdef RID4
#define ROBOT_ID	4
#endif
#ifdef RID5
#define ROBOT_ID	5
#endif
#ifdef RID6
#define ROBOT_ID	6
#endif
#ifdef RID7
#define ROBOT_ID	7
#endif
#ifdef RID8
#define ROBOT_ID	8
#endif
#ifdef RID9
#define ROBOT_ID	9
#endif


enum TestState {NONE = 0, DRIBBLER, KICKER, MOTORS};

// test state functions
enum TestCases {TEST_DRIBBLE_CURRENT = 0, TEST_BALL_SENSOR, TEST_KICKER, TEST_VBATT_LOW, TEST_SERVO, TEST_ADC, TEST_SERVO2, 
		TEST_DRIBBLER, TEST_SIMPLE_SERVO, TEST_MOTORS, TEST_MOTOR, TEST_QDEC, TEST_MOTORS2};

/**************************** PROTOTYPES ***************************/
void ServoTest(void);
void RampVel(int vmax, int comp);
void RunMotor(int i, int maxv, int del);
int GetPacket(Command *command, int ourid);
void RunTest();
void ShowRate(int r, int total);

ushort crcProcess(ushort *buf, int len);
ushort genCrcTable();

const ushort robot_id_table[] = { 0xAA, 0xAB, 0xBA, 0xBB, 0xBC, 0xCB, 0xCC, 0xCD, 0xDC, 0xDD};


// The following table was taken from ID Software's release of the
// Quake 1 source code, released under the GPL:

// This is a 16 bit, non-reflected CRC using the polynomial 0x1021 and
// the initial and final xor values shown below...  in other words,
// the CCITT standard CRC used by XMODEM

ushort crctable[256];

 

/*
 * main -
 *
 * this function ramps up and down each motor driver individually to test its operation
 */
void main(void)
{
  SCIParam sparam;
  ushort led_on = 0;
  BoardParam bp;
  int enable = FALSE;

  int lastpid = 0;
  int lostpid = 0;
  int totalpid = 0;

  Command command;

  /* disable all interrupts */
  DI();

  //  memset((char *) &command, 0, sizeof(command)); 

  /* Inittialise the board */	
  bp.wd_en = FALSE;
  bp.saram_mode = SARAM_BOTH;
  InitBoard(&bp);

  /* setup the serial port */
  sparam.nr_chars = 8;
  sparam.parity   = SCI_NO_PARITY;
  sparam.nr_stop  = 1;
  sparam.baud     = BAUD_RATE;
  sparam.ctrl     = SCI_TXEN | SCI_RXEN;
  InitSCI(&sparam);
 
  /* Initialise the motor drivers */
  EnableServo(enable);
  InitServo();

  // generate the CRC table
  genCrcTable(); 

#ifdef RUNTEST
  RunTest();
#endif

  /* send a message and flash a led to show we are alive */
  LED3_OFF();
  LED2_OFF();
  LED1_OFF();
  FlashLed(SLOW_DELAY, ROBOT_ID + 1);

  /* All go Baby */
  EI();

  /* run servo test if button presssed */
  if (GET_PB0())
    ServoTest();

  LED2_ON();

  for (;;) {


    // show error rate
    if (GET_PB1())
      ShowRate(lostpid, totalpid);     
    
    // get teh latest packet
    GetPacket(&command, ROBOT_ID);

    // reset the servo watchdog
    ResetServoWatchdog();

    // calculate pid loss here
    if (totalpid == 0) {
      lastpid = command.pid;
      totalpid = 1;
    } else {
      if (command.pid > lastpid)
	lostpid += command.pid - lastpid;
      else
	lostpid += 8 + command.pid - lastpid;
      totalpid += lostpid + 1;
    }
    enable = !command.halt;
    
    led_on = !led_on;
    LED1(led_on);

    SetSensorMode(command.use_current_sensing);

    // set the commands
    if (enable) {
      EnableServo(enable);
      EnableKickCharger(command.kcharge);
      SetKick(enable && command.kick);
      SetDribbler(command.drib);
      SetServo(command.rvx, command.rvy, command.rva);  
    } else {
      EnableServo(enable);
      EnableKickCharger(FALSE);
      SetKick(FALSE);
      SetDribbler(0);
      SetServo(0, 0, 0);  
    }
  }
}



// get the packet from the serial port and decode it
int GetPacket(Command *command, int ourid)
{
  enum PacketStage packetstage = PADDING1;
  ushort crcresult; 
  CommPacket cpacket;
  ushort checksum;
  int nopacket = TRUE;
  ushort data, olddata;

  nopacket = TRUE;

  ourid++;

  while (nopacket) {
    data = SCIRecv() & 0xFF;
    switch (packetstage) {
    case PADDING1:
      if (data == 0x00)
	packetstage = PADDING2;
      break;
    case PADDING2:
      if (data == 0xFF)
	packetstage = ROBOTID;
      else if (data != 0)
	packetstage = PADDING2;
      break;
    case ROBOTID:
      cpacket.id = data;
      if (data == ourid)
	packetstage = MAINPACKET;
      else if (data == 0)
	packetstage = PADDING2;
      else
	packetstage = PADDING1;
      break;
    case MAINPACKET:

      cpacket.dx = data;
      cpacket.dy = SCIRecv() & 0xFF;
      cpacket.da = SCIRecv() & 0xFF;
      cpacket.flags = SCIRecv() & 0xFF;
      cpacket.drib_cnt = SCIRecv() & 0xFF;

      // check the check sum and return;
      checksum = 0;
      data = 0;
      olddata = 0;
      olddata = SCIRecv() & 0xFF;
      data = SCIRecv() & 0xFF;
      checksum = (data << 8);
      checksum |= olddata;

      crcresult = crcProcess((ushort *) &cpacket, 6);
      if (crcresult == checksum) {
	nopacket = FALSE;
      } else if (data == 0) {
	packetstage = PADDING2;
      } else if ((data == 0xFF) && (olddata == 0)) {
	packetstage = ROBOTID;
      } else {
	packetstage = PADDING1;
      }

      break;
    }
  }

  //setup the command fields
  command->pid = cpacket.drib_cnt & RF_PID_BITS;

  command->drib = (int) ((cpacket.drib_cnt >> RF_DRIBBLE_SHIFT) & RF_DRIBBLE_MASK);
  command->drib = RVS_FLAG(command->drib, cpacket.flags, RF_REVERSE_DRIBBLE);

  command->rvx = RVS_FLAG((int) cpacket.dx, cpacket.flags, RF_REVERSE_DX);
  command->rvy = RVS_FLAG((int) cpacket.dy, cpacket.flags, RF_REVERSE_DY);
  command->rva = RVS_FLAG((int) cpacket.da, cpacket.flags, RF_REVERSE_DA);

  command->halt = (cpacket.flags & RF_HALT);
  command->use_current_sensing = (cpacket.flags & RF_USE_CURRENT);
  command->kick = (cpacket.flags & RF_KICK);
  command->kcharge = (cpacket.flags & RF_KICKER_CHARGE);

  return (TRUE);
}

ushort crcProcess(ushort *buf, int len)
{
  ushort crcval = 0xFFFF;
  while (len--) {
    crcval = (crcval << 8) ^ crctable[(crcval >> 8) ^ (buf[len] & 0xFF)];
  }
  return (crcval);
}


ushort genCrcTable() 
{
  ushort crcval;
  ushort i, j;

  for (i = 0; i < 256; i++) {
    crcval = i << 8;
    for (j = 0; j < 8; j++) {
      if (crcval & 0x8000) {
	crcval <<= 1;
	crcval ^= 0x1021;
      } else {
	crcval <<= 1;
      }
    }
    crctable[i] = crcval & 0xFFFF;
  }
}

void ShowRate(int r, int total)
{
  LED1_OFF();
  LED2_OFF();
  LED3_OFF();

  // 10's
  SimpleDelay(SLOW_DELAY * 2);
  FlashLed(SLOW_DELAY, ((r * 10) / total));
  SimpleDelay(SLOW_DELAY * 2);
  FlashLed(FAST_DELAY, 30);

  // 1's
  SimpleDelay(SLOW_DELAY * 2);
  FlashLed(SLOW_DELAY, ((r * 100) / total) % 10);
  SimpleDelay(SLOW_DELAY * 2);
  FlashLed(FAST_DELAY, 30);

  // 0.1's
  SimpleDelay(SLOW_DELAY * 2);
  FlashLed(SLOW_DELAY, ((r * 1000) / total) % 10);
  SimpleDelay(SLOW_DELAY * 2);
  FlashLed(FAST_DELAY, 30);
}

void ServoTest(void)
{
  int vel, en = 0;
  int vmax = 250;
  int i;
  enum TestState state = NONE;

  for (i = 1; i <= 3; i++)
    FlashLedi(i, FAST_DELAY, 4);
  for (i = 1; i <= 3; i++)
    FlashLedi(i, FAST_DELAY, 4);

  EnableTimeout(FALSE);

  while (1) {
    //    ResetServoWatchdog();
    switch (state) {
    case NONE:
      SetDribbler(0);
      EnableKickCharger(FALSE);
      EnableServo(FALSE);

      LED1_OFF();
      FlashLedi(1, SLOW_DELAY, 1);
      if (GET_PB0() && !GET_PB1())
	state = DRIBBLER;
      else if (!GET_PB0() && GET_PB1())
	state = KICKER;
      else if (GET_PB0() && GET_PB1())
	state = MOTORS;
      break;
    case DRIBBLER:
      SetDribbler(0xF);

      // should make this use the servo setting
      LED2(ADCReadResult(ADCRES_MTR3) > ADC_CONV(0.75 * 0.02 * (1.0 + 39.0 / 1.0)));
      FlashLedi(1, FAST_DELAY, 10);
      if (GET_PB0())
	state = NONE;
      break;
    case KICKER:
#ifdef USE_CURRENT_SENSING
      SetDribbler(0xF);
#else
      SetDribbler(0);
#endif
      EnableKickCharger(TRUE);

      FlashLedi(1, SLOW_DELAY, 1);
      SetKick(GET_PB1());
      if (GET_PB0())
	state = NONE;
      break;

    case MOTORS:
      FlashLedi(1, SLOW_DELAY, 1);
      ResetServoWatchdog();
      EnableServo(TRUE);
      RampVel(100, 0);
      RampVel(100, 1);
      RampVel(100, 2);

      state = NONE;
      break;
    }
  }
}


// acceleration per 10ms
#define ACC		(100 / (1000 / 100))
//#define ACC		1

void RampVel(int vmax, int comp)
{
  int vel;
  
  for (vel = 0; vel < vmax; vel += ACC) {
    ResetServoWatchdog();
    EnableServo(TRUE);
    if (comp == 0)
      SetServo(vel, 0, 0);
    else if (comp == 1)
      SetServo(0, vel, 0);
    else
      SetServo(0, 0, vel);
    ServoDelay(100);
  }
  for (; vel > -vmax; vel -= ACC) {
    ResetServoWatchdog();
    EnableServo(TRUE);
    if (comp == 0)
      SetServo(vel, 0, 0);
    else if (comp == 1)
      SetServo(0, vel, 0);
    else
      SetServo(0, 0, vel);
    ServoDelay(100);
  }
  for (; vel < 0; vel += ACC) {
    ResetServoWatchdog();
    EnableServo(TRUE);
    if (comp == 0)
      SetServo(vel, 0, 0);
    else if (comp == 1)
      SetServo(0, vel, 0);
    else
      SetServo(0, 0, vel);
    ServoDelay(100);
  }
  SetServo(0, 0, 0);
}


void RunMotor(int i, int maxv, int del)
{
  int	j;
  
  for (j = 0; j < maxv; j += 5) {
    if (i & 0x01)
      SetMotorNew(j, 0, 0, 0);
    if (i & 0x02)
      SetMotorNew(0, j, 0, 0);
    if (i & 0x04)
      SetMotorNew(0, 0, j, 0);
    if (i & 0x08)
      SetMotorNew(0, 0, 0, j);
    SimpleDelay(del);
  }
  for (; j > -maxv; j -= 5) {
    if (i & 0x01)
      SetMotorNew(j, 0, 0, 0);
    if (i & 0x02)
      SetMotorNew(0, j, 0, 0);
    if (i & 0x04)
      SetMotorNew(0, 0, j, 0);
    if (i & 0x08)
      SetMotorNew(0, 0, 0, j);
    SimpleDelay(del);
  }
  for (; j <= 0; j += 5) {
    if (i & 0x01)
      SetMotorNew(j, 0, 0, 0);
    if (i & 0x02)
      SetMotorNew(0, j, 0, 0);
    if (i & 0x04)
      SetMotorNew(0, 0, j, 0);
    if (i & 0x08)
      SetMotorNew(0, 0, 0, j);
    SimpleDelay(del);
  }
}


void RunTest() 
{
  int i = 0, j = 0;
  int maxv = 300;
  int dval = 200;
  ushort enc0 = 0;
  ushort enc1 = 0;
  ushort enc2 = 0;
  int err;
  int pd = 0;
  int led = FALSE;
  int rvx, rvy, rva;

  enum TestCases testcase = TEST_DRIBBLER;

  LED3_OFF();
  LED2_OFF();
  LED1_OFF();
  FlashLedi(3, FAST_DELAY, 5);

  EnableTimeout(FALSE);

  switch (testcase) {
  case TEST_BALL_SENSOR:
    for (;;) {
      BALL_ON();
      LED2_ON();
      LED3(GET_BALL());
      BALL_OFF();
      LED2_OFF();
      SimpleDelay(FAST_DELAY);
    }
    break;
  case TEST_KICKER:
    EI();
    for (;;) {
      ResetServoWatchdog();
      EnableKickCharger(TRUE);
      SetKick(GET_PB1());
      ServoDelay(30);
    }
    break;

  case TEST_VBATT_LOW:
    for (;;) {
      if (GET_VBATT() < ADC_CONV(0.204 * 7.5)) {
	FlashLed(FAST_DELAY, 2);
      }
    }
      
    break;
  case TEST_ADC:

    // test ADC

    LED1_OFF();
    LED2_OFF();
    LED3_OFF();

    SetMotor0(MAX_PWM / 2);

    for (;;) {
      LED1_ON();
      LED1_OFF();
      if (ADCReadResult(ADCRES_MTR0) > ADC_CONV(0.5 * 0.02 * (1.0 + 39.0 / 1.0)))
	LED2_ON();
      else
	LED2_OFF();
      if (ADCReadResult(ADCRES_MTR0) > ADC_CONV(1.0 * 0.02 * (1.0 + 39.0 / 1.0)))
	LED3_ON();
      else
	LED3_OFF();
    }
    break;

    

  case TEST_DRIBBLE_CURRENT:
    LED3_OFF();
    
    FlashLed(FAST_DELAY, ROBOT_ID + 1);
  
    EI();
    EnableServo(FALSE);
    for (;;) {
      //      SetDribbler(DRIB_MAX_PWM);
      SetDribbler(0xF);
      led = (ADCReadResult(ADCRES_MTR3) > ADC_CONV(0.75 * 0.02 * (1.0 + 39.0 / 1.0)));
      LED2(led);
      LED1(led);
      LED3(led);
    }
    break;

  case TEST_DRIBBLER:
    LED1_OFF();
    LED2_OFF();
    LED3_OFF();
    EI();
    EnableServo(FALSE);
    ResetServoWatchdog();

    // test dribbler
    for (;;) {
      SetDribbler(-0xF);
      FlashLedi(1, SLOW_DELAY, 1);
      SetDribbler(0);
      FlashLedi(2, SLOW_DELAY, 1);
      SetDribbler(0xF);
      FlashLedi(3, SLOW_DELAY, 1);
    }
    break;
  case TEST_QDEC:

    j = 1;
    for (;;) {
      enc0 = -ReadQDEC(1);
      enc1 = -ReadQDEC(2);
      enc2 = -ReadEncoder3();
      err = pd - (int) enc0;
      err >>= 3;
      err = BOUND(err, maxv, -maxv);
      SetMotor0(err);

      err = pd - (int) enc1;
      err >>= 3;
      err = BOUND(err, maxv, -maxv);
      SetMotor1(err);

      err = pd - (int) enc2;
      err >>= 3;
      err = BOUND(err, maxv, -maxv);
      SetMotor2(err);

      if ((i++) % 0xF == 0)
	pd += j;
      if (pd > 5000)
	j = -1;
      else if (pd <= 0)
	j = 1;
    }
    break;

  case TEST_SERVO:
    EI();
    for (;;) {
      LED3_ON();
      EnableServo(TRUE);
      rvx = 0;
      rvy = 0;
      rva = 20;
      SetServo(rvx, rvy, rva);
    }
  break;

  case TEST_SERVO2:
    EI();
    maxv = 0;
    dval = 10;
    for (;;) {
      maxv += dval;
      if ((maxv > 500) || (maxv < -500))
	dval = -dval;
    
      EnableServo(TRUE);
      rvx = 0;
      rvy = 0;
      rva = maxv;
      SetServo(rvx, rvy, rva);
      //    SetServo(0, 0, 0);
      //      LED3(currentreached);
      //      LED2(limiting);
      FlashLedi(1, FAST_DELAY, 1);
    }

    break;


  case TEST_SIMPLE_SERVO:
    for (;;) {
      SetMotorNew(maxv, maxv, maxv, maxv);

      enc0 = ReadQDEC(2);
      if (enc1 - enc0 > 0)
	LED2_ON();
      else
	LED2_OFF();
      enc1 = enc0;

      SimpleDelay(100);
    }

    break;
  case TEST_MOTOR:
    LED1_OFF();
    LED2_OFF();
    LED3_OFF();
    SimpleDelay(2000);
    
    for (;;) {
      SetMotor3(MAX_PWM / 2);
      LED1_ON();
      SimpleDelay(5000);
      LED1_OFF();

      SetMotor3(-MAX_PWM / 2);
      LED2_ON();
      SimpleDelay(5000);
      LED2_OFF();
    }

    break;
  case TEST_MOTORS:
    for (;;) {
      LED1_OFF();
      LED2_OFF();
      LED3_OFF();
      led = !led;
      for (i = -maxv; i < maxv; i++) {
	KickDog();
	SetMotorNew(i, i, i, i);
	LED1(led);
	LED2(!led);
	SimpleDelay(100);
      }
      led = !led;
      for (; i > -maxv; i--) {
	KickDog();
	SetMotorNew(i, i, i, i);
	LED1(led);
	LED2(!led);
	SimpleDelay(100);
      }
    }
    break;

  case TEST_MOTORS2:
    maxv = 300;

    for (;;) {
      LED1_OFF();
      LED2_OFF();
      LED3_OFF();
      led = !led;
      for (i = -maxv; i < maxv; i++) {
	KickDog();
	SetMotorNew(i, i, i, 0);
	LED1(led);
	LED2(!led);
	SimpleDelay(100);
      }
      led = !led;
      for (; i > -maxv; i--) {
	KickDog();
	SetMotorNew(i, i, i, 0);
	LED1(led);
	LED2(!led);
	SimpleDelay(100);
      }
    }
    break;
  }
}
