/*
 * TITLE:		servo.c
 *
 * DESCRIPTION:	This file contains the servo routines for the Omnibot and DiffBot robots.
 *				It contains three main entry points:
 *				InitServo		- initialises the servo interrupt driver and variables
 *				Timer_Isr		- servo interrupt routine running of the EVx timer interupt
 *				SetServo		- sets the set points for the servo loop
 *				EnableServo		- enables or disables the servo routine for power saving operations
 *
 *				The servo routine runs a standard discrete time PID loop. Look to Applicationa note
 *				SPRA083 Application report form www.ti.com for more information or any control book.
 *
 * WRITTEN BY:	Brett Browning
 *
 * REVISION HISTORY:
 * 5 April 2001		- Created (BB)
 * 11 June 2001		- Ported from servo test routines
 */
/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#include "lf2407.h"
#include "servo.h"
#include "board.h"
#include "evx.h"

//#include "current-limiting.h"

//#define SHOW_LED
//#define USE_ROTATE

/**************************** TYPES ********************************/

/* encoder channels */
#define LENC_CH			1
#define RENC_CH			2

/* encoder channels */
#define W1ENC_CH		1
#define W2ENC_CH		2


/* bits for interrupts */
#define T1PINTE		0x80
#define T1PINTR		0x02

/* ISR counts between servo execution */
#define SERVO_TICK_LENGTH	1

// encoder counts per revolution -- with quadrature
#define COUNTPERREV		(512.0 * 4.0)

// make sure you measure this while running from flash!!!
#define SERVO_FREQ		(1028.0/2.0) // Hz
#define WHEEL_DIAMETER		4.0  // cm
#define WHEEL_BASE		6.35 //cm

#define GEAR_RATIO		9.7  // : 1

/* macro for converting P,D,I into K1, K2 an dK3 for digital PID */
#define TPER			(1.0 / SERVO_FREQ)

/* Just to see if thise makes a difference, ~8mm each */
#define IX_LIMIT		2000L
#define IY_LIMIT		2000L
#define IA_LIMIT		2000L
#define IV_LIMIT		2000L

/* EVx timer channel settings - deadband time in ns */
#define DEAD_BAND		300L // ns

/* timeout for radio commands ~  */
#define COMMAND_TIMEOUT		500L //ms

// timeout for VBATT low LED's
#define VBATT_TIMEOUT      500 // ms

// current ~0.75A * resistor network
#define DRIBBLER_CURRENT_THRESH    (0.75 * 0.02 * (1.0 + 39.0 / 1.0))

// motor current threshold = maxcurrent * R_sens * Opamp gain
/* Max allowable current to any given motor */
#define MOTOR_MAX_CURR  1.8 //2.0 //1.20  // Amps
#define MTR_CURRENT_THRESH         (MOTOR_MAX_CURR * 0.02 * (1.0 + 39.0 / 1.0))

// This needs to be sufficiently low that the motors will not burn out even in stall
// from micromo data sheets, with 8V nominal supply, we need to limit current to 1.2A
// with R = 1.94, this gives V = 2.38....we will approximate this very roughly
#define MAX_OVERLOAD_PWM  ((ushort) (MAX_PWM / 3))

/**************************** PROTOTYPES ***************************/
void delay(ulong d);
inline void ServoOmni(void);
inline void OmniForward(long *w1, long *w2, long *w3, long vx, long vy, long w);
inline void OmniInverse(long *vx, long *vy, long *w, long w1, long w2, long w3);

/**************************** GLOBASL *******************************/
ulong timer = 0;
int clk_val = 0;
int on_val = 0;
int stuck_flag = 0;
ulong command_timeout = 0;

int timeoutenabled = 1;

// kicker variables
enum KickerState {KFLAT = 0, KICKING, KICKOFF, KCHARGING, KCHARGEOFF, KSWITCH_TO_KICK};

// kicking variables
// kicking timeouts for states -- in ms
//                               //kflat  kicking  kickoff kcharging  kchargeoff  kswitch_to_kick
const ulong kicking_timeouts[] =  {0,     50,      50,      0,         2,          2};
ulong ktimeout = 0;
enum KickerState kickerstate = KFLAT;
int kchargeren = 0;
int kicknow = 0;
int kicking = 0;

// dribbler variables
int dribbleron = 0;
int ballondribbler = 0;
int drib_pwm = 0;

/* servo loop variables -
 * xv			- encoder velocities for left and right
 * xvd			- desired velocity value (forward and dv)
 * vxcmdy		- velocity command values for x (forward, rotational) and delay y (_, 1, 2, 3)
 * xerry		- error term for x (forward, rotational) and delay y (_, 1, 2, 3)
 * xency		- encoder values and 
 */
int servo_enabled = FALSE;

// wheel velocities
long v1 = 0, v2 = 0, v3 = 0;
ushort w1enc0 = 0, w1enc1 = 0;
ushort w2enc0 = 0, w2enc1 = 0;
ushort w3enc0 = 0, w3enc1 = 0;
short v1cmd = 0, v2cmd = 0, v3cmd = 0;

long vx = 0, vy = 0, va = 0;
long vxd = 0, vyd = 0, vad = 0;

long vxcmd = 0, vycmd = 0, vacmd = 0;
long vxerr = 0, vyerr = 0, vaerr = 0;

long ixerr = 0, iyerr = 0, iaerr = 0;
long dxerr = 0, dyerr = 0, daerr = 0;

// data for toggling led
int led3state = 0;
ulong led3timeout = 0;

int limiting = 0;
int currentreached = 0;
long ilimit = 100000L;
short pwmlim = 0;

// flag for controlling current sendsing
int use_current_sensing = 0;

/**************************** CODE *******************************/

// This function intialises the timer channel interrupt and the EVx modules. Interrupts need
// to be enabled to complete the operation but this is left to the main routines.
int InitServo(void)
{
  int i;

  /* disable all interrupts */
  DI();

  /* set a default motor command */
  //  SetMotorNew(0, 0, 0, 0);

  /* setup the EV units for PWM and Quadrature decode */
  // 14000 gives a ~19kHz clock with 40MHz oscillator
  InitEvx(0);
  SetGPTimer(1, 14000);
  //  InitDBPWM(EVA_CHANNELS | EVB_CHANNELS, 0x333, MAX_PWM, DEAD_BAND);

  // are our level shifters inverters or not?
#ifdef INVMOTOR
  InitDBPWM(MTR_CHANNELS, EVX_CH7 | EVX_CH8 | EVX_CH9 | EVX_CH10 | 
	    EVX_CH3 | EVX_CH4 | EVX_CH11 | EVX_CH12, MAX_PWM, DEAD_BAND);
#else
  InitDBPWM(MTR_CHANNELS, 0, MAX_PWM, DEAD_BAND);
#endif

  /* set a default motor command */
  SetMotorNew(0, 0, 0, 0);

  InitQDEC(1);
  InitQDEC(2);

  /* clear the interrupt mask */
  INTR_REGS.imr = T1PINTR;
  EVA_REGS.intrregs.imra = T1PINTE;
  
  /* clear first flag */
  EVA_REGS.intrregs.ifra |= T1PINTE;
}


//----------------- setting functions ----------------------

inline void SetSensorMode(int use_current)
{
  use_current_sensing = use_current;
}

inline void EnableTimeout(int en)
{
  timeoutenabled = en;
}

void SetDribbler(int speed)
{
  // dribpwm varies to +/- 16 = 4 bits
  // shift to +/-1024 = 10 bits
  drib_pwm = (speed << 6);
  drib_pwm = BOUND(drib_pwm, MAX_PWM, -MAX_PWM);
  dribbleron = !(drib_pwm == 0);
}

inline void SetKick(int _kick)
{
  kicknow = _kick;
}

inline void EnableKickCharger(int en)
{
  kchargeren = en;
}

// this funciton implements the more accurate delay routine by using the system clock
void ServoDelay(ulong d)
{
  d += timer;
  while (timer < d) {
    /* reset the command timeout */
    ResetServoWatchdog();
  }
}

inline void ResetServoWatchdog()
{
  command_timeout = timer + COMMAND_TIMEOUT;
}


// this function sets teh set points for the servo routine
void SetServo(int vxset, int vyset, int vaset)
{
  // this needs fixing...
  vxd = (((long) vxset) * 422) >> 7;
  vyd = (((long) vyset) * 422) >> 7;
  //  dvd = (tmpdvd * 162) >> 7;
  vad = (((long) vaset) * 162) >> 5; 	
}

// This function enables the servo routines to output to the motor velocities
void EnableServo(int en)
{
  /* clear out the prior variables to enable a nice restart */
  servo_enabled = en;
  if (!en) {
    vxcmd = 0;
    vycmd = 0;
    vacmd = 0;
    vxerr = 0;
    vyerr = 0;
    vaerr = 0;
    ixerr = 0;
    iyerr = 0;
    iaerr = 0;
  }
  ResetServoWatchdog();
}

// This function resets the integral errors on the servo routines
void ResetIntError(void)
{
  ixerr = 0;
  iyerr = 0;
  iaerr = 0;
}


// --------------- kicker code ------------------------
  


// kicker state machine. We run this every servo loop. We basically want to kick whenever the kick
// command is sent and the dribbler state is okay. We want to charge if charge is enabled, and we are not
// kicking. We want a delay between switching the charging state and kicking and vice versa.
// State machine looks like:
// state     output    timeout          conditions     transition
// KFLAT     0         0                kicking                 KICKING
//                                      chargeenable            KCHARGING
// KICKING   kick      yes              -                       KICKOFF
// KICKOFF   0         yes              -                       KFLAT
// KCHARGING kcharge   0                !chargeenable           KCHARGEOFF
//                                      kicking                 KCHARGEOFF
// KCHARGEOFF 0        yes              kicking                 KICKING
//                                      -                       KFLAT
void runKicker()
{
  if (ktimeout > 0)
    ktimeout--;

  switch (kickerstate) {
  case KFLAT:
    LED1_OFF();
    LED2_OFF();
    LED3_OFF();

    KICK_OFF();
    KEN_OFF();
    if (kicking) {
      kickerstate = KICKING;
      ktimeout = kicking_timeouts[kickerstate];
    } else if (kchargeren) {
      kickerstate = KCHARGING;
      ktimeout = kicking_timeouts[kickerstate];
    }
    break;
  case KICKING:
    LED1_OFF();
    LED2_ON();
    LED3_OFF();

    KICK_ON();
    KEN_OFF(); // should be off, but lets make sure

    // must kick for a set time
    if (ktimeout == 0) {
      kickerstate = KICKOFF;
      ktimeout = kicking_timeouts[kickerstate];
    }
    break;
  case KICKOFF:
    LED1_ON();
    LED2_ON();
    LED3_OFF();

    KICK_OFF();
    KEN_OFF();
    if (ktimeout == 0) {
      kickerstate = KFLAT;
      kicking = 0;
      ktimeout = kicking_timeouts[kickerstate];
    }
    break;
  case KCHARGING:
    LED1_OFF();
    LED2_OFF();
    LED3_ON();

    KICK_OFF(); // should be off, but lets make sure
    KEN_ON(); 
    if (kicking) {
      kickerstate = KSWITCH_TO_KICK;
      ktimeout = kicking_timeouts[kickerstate];
    } else if (!kchargeren) {
      kickerstate = KCHARGEOFF;
      ktimeout = kicking_timeouts[kickerstate];
    }
    break;

  case KCHARGEOFF:
    LED1_ON();
    LED2_OFF();
    LED3_ON();

    KICK_OFF();
    KEN_OFF();

    if (ktimeout == 0) {
      kickerstate = KFLAT;
      ktimeout = kicking_timeouts[kickerstate];
    }
    break;

  case KSWITCH_TO_KICK:
    KICK_OFF();
    KEN_OFF();
    if (ktimeout == 0) {
      kickerstate = KICKING;
      ktimeout = kicking_timeouts[kickerstate];
    }
    break;
  }

}


// ------------------- servo code -----------------------

/*
 * Servo_ISR -
 *
 * Servo interrupt routine. This is called from the general ISR for the EVX module (See Vectors.asm)
 * at the end of each PWM period at 14.4kHz. It down samples this rate to the servo routine rate
 * and executes the appropriate servo code.	The servo routines are discrete time PID equations as
 * per the application note specified above
 */
void interrupt Servo_ISR(void)
{

  /* achieve a 1kHz signal rate by checking what loop we are on */
  // base ISR rate is PWM period, now at 19.53kHz
  // result is ~1028Hz
  if (clk_val++ > 13) {

    clk_val = 0;
    KickDog();
    timer++;

    // work out if ball is on dribbler
    if (use_current_sensing) {
      ballondribbler = !dribbleron || (ADCReadResult(ADCRES_MTR3) > ADC_CONV(DRIBBLER_CURRENT_THRESH));
    } else {
      BALL_ON();
      ballondribbler = GET_BALL();
      BALL_OFF();
    }

    // set our kicking flag
    kicking = kicknow && ballondribbler;

    // do the dirty kicker business
    runKicker();

    // check battery voltage -- low is 7V
    if (GET_VBATT() < ADC_CONV(0.204 * 7.0)) {
      if (timer > led3timeout) {
	led3state = !led3state;
	led3timeout = timer + VBATT_TIMEOUT;
      }
      //      LED3(led3state);
    } else if (led3state) {
      led3state = 0;
      //      LED3_OFF();
    }
  
    /* if we haven't received a command in a while then shut down the servo loop */
    if (timeoutenabled && (timer > command_timeout)) {
      servo_enabled = FALSE;
      kchargeren = FALSE;
      kicknow = FALSE;
      drib_pwm = 0;
    }
    
    /* run the diffbot servo code and update ht erobot heading */
    // actual servo code
    // omni runs at 485Hz
    if (timer & 1)
      ServoOmni();
  }
#ifdef SHOW_LED
  LED_OFF();
#endif
  
  /* clear the interrupt mask and the EVx interrupt mask and reneable interrupts */	
  EVA_REGS.intrregs.ifra |= T1PINTE;
  INTR_REGS.imr |= T1PINTR;
  EI();
}


// The Servo loop for the omni
inline void ServoOmni(void)
{
  long	tmpval;
  long	tmpv1cmd, tmpv2cmd, tmpv3cmd;
  short	tmpv1, tmpv2, tmpv3;
  int i;

  if (!servo_enabled) {
    SetMotorNew(0, 0, 0, drib_pwm);
    ixerr = 0; 
    iyerr = 0; 
    iaerr = 0;
    vxerr = 0; vyerr = 0; vaerr = 0;
    vxcmd = 0; vycmd = 0; vacmd = 0;
    stuck_flag = FALSE;
    
    vxd = 0; vyd = 0; vad = 0;
    return;
  }
  
  /* read the encoders - note right encoder is reversed */
  /* need to put wrap(mtrlimit[0] > CURRENT_THRE) checkhere */
  w1enc0 = ReadQDEC(W1ENC_CH);
  v1 = (short) (w1enc1 - w1enc0);
  w1enc1 = w1enc0;
  
  w2enc0 = ReadQDEC(W2ENC_CH);
  v2 = (short) (w2enc1 - w2enc0);
  w2enc1 = w2enc0;
  
  w3enc0 = ReadEncoder3();
  v3 = (short) (w3enc1 - w3enc0);
  w3enc1 = w3enc0;
 
  /* watch out for wrapping. So lets bound it down 
   * - need to make sure ISR runs fast enough that we don't hit this
   * limit
   */
  v1 = (long) BOUND(v1, 0x1000, -0x1000);
  v2 = (long) BOUND(v2, 0x1000, -0x1000);
  v3 = (long) BOUND(v3, 0x1000, -0x1000);

  // do the inverse transform to find the robot velocity components
  OmniInverse(&vx, &vy, &va, v1, v2, v3);
  
  /* calculate velocity errors */
  vxerr = vxd - vx;
  vyerr = vyd - vy;
  vaerr = vad - va;
  
  /* calcluate integral error */
  ixerr += vxerr;
  iyerr += vyerr;
  iaerr += vaerr;
  
  /* check for saturation by looking at large integral errors and bound it */
  stuck_flag = ((ABS(ixerr) > IX_LIMIT) || (ABS(iyerr) > IY_LIMIT) || (ABS(iaerr) > IA_LIMIT));
 
  ixerr = BOUND(ixerr, IX_LIMIT, -IX_LIMIT);
  iyerr = BOUND(iyerr, IY_LIMIT, -IY_LIMIT);
  iaerr = BOUND(iaerr, IA_LIMIT, -IA_LIMIT);

  vxcmd = ((5 * vxerr + 20 * ixerr) >> 4);
  vycmd = ((5 * vyerr + 20 * iyerr) >> 4);
  //  vacmd = (20 * vaerr + 20 * iaerr) >> 5;
  vacmd = (10 * vaerr + 70 * iaerr) >> 5;
  //  vacmd = (5 * vaerr + 90 * iaerr) >> 5;

  //  vacmd = (20 * vaerr + 50 * iaerr) >> 5; // -- normal value
  //  vacmd = (32 * vaerr + 70 * iaerr) >> 5; // -- bit faster but crazier

  // do the forward transform to get back into wheel velocities	
  OmniForward(&tmpv1cmd, &tmpv2cmd, &tmpv3cmd, vxcmd, vycmd, vacmd);

  // are we exceeding the current threshold?
  currentreached = (ADCReadResult(ADCRES_MTR0) > ADC_CONV(MTR_CURRENT_THRESH)) ||
    (ADCReadResult(ADCRES_MTR1) > ADC_CONV(MTR_CURRENT_THRESH)) ||
    (ADCReadResult(ADCRES_MTR2) > ADC_CONV(MTR_CURRENT_THRESH));

  // if we are, we decrease the pwmlimit until current drops
  // this will take ~2s (1024 ISRs at 570Hz) in worst case
  // we have fast recovery model
  if (currentreached) {
    LED3_ON();
    if (pwmlim > MAX_PWM/3) {
      pwmlim -= 4;
      //pwmlim -= (pwmlim >> 2);
    }
  } else {
    LED3(led3state);
    if (pwmlim < MAX_PWM) {
      //pwmlim += ((MAX_PWM - pwmlim) >> 1);
      pwmlim += 4;
    }
  }

  limiting = (ABS((short) tmpv1cmd) > pwmlim) ||
    (ABS((short) tmpv2cmd) > pwmlim) ||
    (ABS((short) tmpv3cmd) > pwmlim);

  // first bound to MAX_PWM to prevent rollovers
  tmpv1cmd = BOUND(tmpv1cmd, MAX_PWM, -MAX_PWM);
  tmpv2cmd = BOUND(tmpv2cmd, MAX_PWM, -MAX_PWM);
  tmpv3cmd = BOUND(tmpv3cmd, MAX_PWM, -MAX_PWM);

  //hack to disable
  //pwmlim = MAX_PWM;

  // now scale by pwmlim accounting for scale of 1024
  tmpv1cmd *= pwmlim;
  tmpv1cmd >>= 10;
  tmpv2cmd *= pwmlim;
  tmpv2cmd >>= 10;
  tmpv3cmd *= pwmlim;
  tmpv3cmd >>= 10;

  v1cmd = (short) tmpv1cmd;
  v2cmd = (short) tmpv2cmd;
  v3cmd = (short) tmpv3cmd;

  //  v1cmd = (short) BOUND((short) tmpv1cmd, MAX_PWM, -MAX_PWM);
  //  v2cmd = (short) BOUND((short) tmpv2cmd, MAX_PWM, -MAX_PWM);
  //  v3cmd = (short) BOUND((short) tmpv3cmd, MAX_PWM, -MAX_PWM);

  /* send it to the motor if enabled */
  SetMotorNew(v1cmd, v2cmd, v3cmd, drib_pwm);
}


inline void OmniForward(long *w1, long *w2, long *w3, long vx, long vy, long w)
{
  long a, b, c;
  a = 222L * vx;
  b = vy * 128;

  // Note the extra precision (was lsh 8)
  c = w << 6;

  // done the long way because compiler screws up w
  *w1 = (b - a + c) >> 8;
  *w2 = w - vy;
  *w3 = (a + b + c) >> 8;
}

inline void OmniInverse(long *vx, long *vy, long *w, long w1, long w2, long w3)
{
  long a;
  a = 85L * (w1 + w3);
  *vx = (148L * (w3 - w1)) >> 8;
  *vy = (a - 171L * w2) >> 8;
  
  // extra precision -- was rsh 8
  *w = (a + 85L * w2) >> 6;
}


#if 0

/*
 * ServoDiff -
 *
 * Ther servo loop function for the diffbot
 */
inline void ServoDiff(void)
{
  /* read the encoders - note right encoder is reversed */
  lenc0 = ReadQDEC(LENC_CH);
  renc0 = ReadQDEC(RENC_CH);


#ifdef PROTOTYPE
  lv = (lenc0 - lenc1);
#else
  lv = (lenc1 - lenc0);
#endif
 
  lenc1 = lenc0;


#ifdef PROTOTYPE
  rv = (renc1 - renc0);
#else
  rv = (renc0 - renc1);
#endif
  renc1 = renc0;


  /* bound to prevent wrap around */
  lv = (long) BOUND(lv, 0x4000, -0x4000);
  rv = (long) BOUND(rv, 0x4000, -0x4000);

#ifdef MEGAROBOT
  lv = -lv;
  rv = -rv;
#endif

  /* translate to robot velocity components */
  vx = (lv + rv) >> 1;
  dv = (rv - lv) >> 1;

  if (servo_enabled) {
    
    /* calculate the error terms */
    vxerr0 = (long) (vxd - vx);
    derr0 = (long) (dvd - dv);
    
    /* calcluate integral errors for saturation test */
    ixerr += vxerr0;
    iderr += derr0;

    /* calculate derivative errors */
    dxerr = vxerr0 - vxerr1;
    dderr = derr0 - derr1;
		
    vxerr1 = vxerr0;
    derr1 = derr0;

    /* check for saturation by looking at large integral errors */
    stuck_flag = ((ABS(ixerr) > IX_LIMIT) || (ABS(iderr) > ID_LIMIT));
    
    ixerr = BOUND(ixerr, IX_LIMIT, -IX_LIMIT);
    iderr = BOUND(iderr, ID_LIMIT, -ID_LIMIT);
    
    /* calculate the PID commands and bound them */	
#ifdef PROTOTYPE
    //		vxcmd0 = (16 * vxerr0 + 32 * ixerr + dxerr >> 1) >> 2;
    //		dvcmd0 = (16 * derr0 + 32 * iderr + dderr >> 1) >> 2;
    vxcmd0 = (32 * vxerr0 + 16 * ixerr);
    dvcmd0 = (32 * derr0 + 16 * iderr);
#else
    // was 6 and 14
    vxcmd0 = (20 * vxerr0 + 14 * ixerr) >> 3;
    dvcmd0 = (20 * derr0 + 14 * iderr) >> 3;
    //		vxcmd0 = (20 * vxerr0 + 14 * ixerr) + vxd >> 6;
    //		dvcmd0 = (20 * derr0 + 20 * iderr) + dvd >> 6;
#endif

    /* transform back into wheel velocities */
    lvcmd = (int) BOUND(vxcmd0 - dvcmd0, MAX_PWM, -MAX_PWM);
    rvcmd = (int) BOUND(vxcmd0 + dvcmd0, MAX_PWM, -MAX_PWM);

#ifdef MEGAROBOT
    lvcmd = -lvcmd;
    //    rvcmd = -rvcmd;
#endif
    
    /* send it to the motor if enabled */
    SetMotorNew(lvcmd, rvcmd, 0, drib_pwm);
  } else {
    SetMotorNew(0, 0, 0, drib_pwm);
    ixerr = 0; iyerr = 0; iderr = 0;
    vxerr0 = 0; vyerr0 = 0; derr0 = 0;
    vxcmd0 = 0; vycmd0 = 0; dvcmd0 = 0;
    stuck_flag = FALSE;
  }
  
  /* shift all the old values across */
  vxcmd2 = vxcmd1;
  vxcmd1 = vxcmd0;
  vxerr2 = vxerr1;
  vxerr1 = vxerr0;
  dvcmd2 = dvcmd1;
  dvcmd1 = dvcmd0;
  derr2 = derr1;
  derr1 = derr0;
}
#endif
