/*
 * TITLE:		servo.h
 *
 * DESCRIPTION:	This file contains the servo routines for 
 *              the Omnibot and DiffBot robots.
 * It contains three main entry points:
 * InitServo	- initialises the servo interrupt driver and variables
 * Timer_Isr	- servo interrupt routine running of the EVx timer interupt
 * SetServo	- sets the set points for the servo loop
 * EnableServo	- enables or disables the servo routine for power saving operations
 *
 * The servo routine runs a standard discrete time PID loop. Look to Applicationa note
 * SPRA083 Application report form www.ti.com for more information or any control book.
 *
 * WRITTEN BY:	Brett Browning
 */
/* LICENSE:
  =========================================================================
    CMDragons'03 RoboCup F180 Firmware Source Code Release
  -------------------------------------------------------------------------
    Copyright (C) 2003 Manuela Veloso, Brett Browning, Mike Bowling,
                       James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
    School of Computer Science, Carnegie Mellon University
  -------------------------------------------------------------------------
    This software is distributed under the GNU General Public License,
    version 2.  If you do not have a copy of this licence, visit
    www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
    Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY,
    including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  ------------------------------------------------------------------------- */
#ifndef _SERVO_H
#define _SERVO_H

#include "lf2407.h"

// determine what we are running as and set the interrupt flag
#ifdef LINUX
#define INTERRUPT
#else
#define INTERRUPT interrupt
#endif


// robot velocity structure
typedef struct tagRobotVel {
  long vx, vy, va;
} RobotVel;

// maximum pwm dribbler speed
#define DRIB_MAX_PWM   16


/**************************** PROTOTYPES *****************************/


/*
 * InitServo -
 *
 * This function intialises the timer channel interrupt and the EVx modules. Interrupts need
 * to be enabled to complete the operation but this is left to the main routines.
 */
int InitServo(void);

// dribbler functions
void SetDribbler(int speed);

// set whether we use current sensing or ball sensors
inline void SetSensorMode(int use_current);

// kicker functions
inline void SetKick(int _kick);
inline void EnableKickCharger(int en);
inline void EnableTimeout(int en);

// timing function using interrupt routine counted time
// this funciton implements the more accurate delay routine 
// by using the system clock. Requires servo to run
void ServoDelay(ulong d);

// servo operations
void SetServo(int vxset, int vyset, int dvset);
void EnableServo(int en);

// reset the integral error
void ResetIntError(void);

// watchdog for command timeouts
void ResetServoWatchdog();

/*
 * Servo_ISR -
 *
 * Servo interrupt routine. This is called from the general ISR for the EVX module (See Vectors.asm)
 * at the end of each PWM period at 14.4kHz. It down samples this rate to the servo routine rate
 * and executes the appropriate servo code.	The servo routines are discrete time PID equations as
 * per the application note specified above
 * this is needed in order to setup the interrupt routine
 */
void interrupt Servo_ISR(void);

#endif /* _SERVO_H */
